@@ -1,4 +1,3 @@
-from __future__ import absolute_import, division, print_function, unicode_literals
 from functools import partial
 import errno
 import os
@@ -21,7 +20,7 @@ STATUS = 0
 STDOUT = 1
 STDERR = 2
 
-# Object ID / SHA1-related constants
+# Object ID / SHA-1-related constants
 # Git's empty tree is a built-in constant object name.
 EMPTY_TREE_OID = '4b825dc642cb6eb9a060e54bf8d69288fbee4904'
 # Git's diff machinery returns zeroes for modified files whose content exists
@@ -57,7 +56,6 @@ def is_git_dir(git_dir):
                 and core.isfile(join(git_dir, 'commondir'))
             )
         ):
-
             result = core.isfile(headref) or (
                 core.islink(headref) and core.readlink(headref).startswith('refs/')
             )
@@ -98,7 +96,7 @@ def read_git_file(path):
     return result
 
 
-class Paths(object):
+class Paths:
     """Git repository paths of interest"""
 
     def __init__(self, git_dir=None, git_file=None, worktree=None, common_dir=None):
@@ -113,7 +111,7 @@ class Paths(object):
         ceiling_dirs = set()
         ceiling = core.getenv('GIT_CEILING_DIRECTORIES')
         if ceiling:
-            ceiling_dirs.update([x for x in ceiling.split(':') if x])
+            ceiling_dirs.update([x for x in ceiling.split(os.pathsep) if x])
 
         if path:
             path = core.abspath(path)
@@ -177,7 +175,7 @@ def find_git_directory(path):
     ).get(path)
 
 
-class Git(object):
+class Git:
     """
     The Git class manages communication with the Git binary
     """
@@ -285,7 +283,7 @@ class Git(object):
         # Start the process
         # Guard against thread-unsafe .git/index.lock files
         if not _readonly:
-            _index_lock.acquire()  # pylint: disable=consider-using-with
+            _index_lock.acquire()
         try:
             status, out, err = core.run_command(
                 command,
@@ -295,7 +293,7 @@ class Git(object):
                 stdout=_stdout,
                 stderr=_stderr,
                 no_win32_startupinfo=_no_win32_startupinfo,
-                **extra
+                **extra,
             )
         finally:
             # Let the next thread in
@@ -310,7 +308,7 @@ class Git(object):
 
         cola_trace = GIT_COLA_TRACE
         if cola_trace == 'trace':
-            msg = 'trace: %.3fs: %s' % (elapsed_time, core.list2cmdline(command))
+            msg = f'trace: {elapsed_time:.3f}s: {core.list2cmdline(command)}'
             Interaction.log_status(status, msg, '')
         elif cola_trace == 'full':
             if out or err:
@@ -323,7 +321,7 @@ class Git(object):
                     '# %.3fs: %s -> %d' % (elapsed_time, ' '.join(command), status)
                 )
         elif cola_trace:
-            core.print_stderr('# %.3fs: %s' % (elapsed_time, ' '.join(command)))
+            core.print_stderr('# {:.3f}s: {}'.format(elapsed_time, ' '.join(command)))
 
         # Allow access to the command's status code
         return (status, out, err)
@@ -331,7 +329,7 @@ class Git(object):
     def git(self, cmd, *args, **kwargs):
         # Handle optional arguments prior to calling transform_kwargs
         # otherwise they'll end up in args, which is bad.
-        _kwargs = dict(_cwd=self.getcwd())
+        _kwargs = {'_cwd': self.getcwd()}
         execute_kwargs = (
             '_cwd',
             '_decode',
@@ -364,8 +362,8 @@ class Git(object):
             result = self.execute(call, **_kwargs)
         except OSError as exc:
             if WIN32 and exc.errno == errno.ENOENT:
-                # see if git exists at all. on win32 it can fail with ENOENT in
-                # case of argv overflow. we should be safe from that but use
+                # see if git exists at all. On win32 it can fail with ENOENT in
+                # case of argv overflow. We should be safe from that but use
                 # defensive coding for the worst-case scenario. On UNIX
                 # we have ENAMETOOLONG but that doesn't exist on Windows.
                 if _git_is_installed():
@@ -377,7 +375,7 @@ class Git(object):
 
 def _git_is_installed():
     """Return True if git is installed"""
-    # On win32 Git commands can fail with ENOENT in case of argv overflow. we
+    # On win32 Git commands can fail with ENOENT in case of argv overflow. We
     # should be safe from that but use defensive coding for the worst-case
     # scenario. On UNIX we have ENAMETOOLONG but that doesn't exist on
     # Windows.
@@ -417,10 +415,10 @@ def transform_kwargs(**kwargs):
         # isinstance(False, int) is True, so we have to check bool first
         if isinstance(value, bool):
             if value:
-                args.append('%s%s' % (dashes, dashify(k)))
+                args.append(f'{dashes}{dashify(k)}')
             # else: pass  # False is ignored; flag=False inhibits --flag
         elif isinstance(value, types_to_stringify):
-            args.append('%s%s%s%s' % (dashes, dashify(k), equals, value))
+            args.append(f'{dashes}{dashify(k)}{equals}{value}')
 
     return args
 
