@@ -1,5 +1,4 @@
 """Launcher and command line interface to git-cola"""
-from __future__ import absolute_import, division, print_function, unicode_literals
 import argparse
 import sys
 
@@ -13,7 +12,7 @@ def main(argv=None):
     app.initialize()
     if argv is None:
         argv = sys.argv[1:]
-    # we're using argparse with subparser, but argparse
+    # we're using argparse with subparsers, but argparse
     # does not allow us to assign a default subparser
     # when none has been specified.  We fake it by injecting
     # 'cola' into the command-line so that parse_args()
@@ -28,10 +27,6 @@ def main(argv=None):
     return args.func(args)
 
 
-def winmain():
-    return app.winmain(main)
-
-
 def parse_args(argv):
     parser = argparse.ArgumentParser()
     # Newer versions of argparse (Python 3.6+) emit an error message for
@@ -147,12 +142,16 @@ def add_dag_command(subparser):
         help='visualize all branches',
         default=False,
     )
-    parser.add_argument('args', nargs='*', metavar='<args>', help='git log arguments')
+    parser.add_argument(
+        'args', nargs=argparse.REMAINDER, metavar='<args>', help='git log arguments'
+    )
 
 
 def add_diff_command(subparser):
     parser = add_command(subparser, 'diff', 'view diffs', cmd_diff)
-    parser.add_argument('args', nargs='*', metavar='<args>', help='git diff arguments')
+    parser.add_argument(
+        'args', nargs=argparse.REMAINDER, metavar='<args>', help='git diff arguments'
+    )
 
 
 def add_fetch_command(subparser):
@@ -349,6 +348,12 @@ def add_rebase_command(subparser):
         action='store_true',
         help="passed to 'git apply' by 'git rebase'",
     )
+    parser.add_argument(
+        '--update-refs',
+        default=False,
+        action='store_true',
+        help='update branches that point to commits that are being rebased',
+    )
     parser.add_argument(
         '-C',
         dest='context_lines',
@@ -438,20 +443,23 @@ def add_tag_command(subparser):
 
 def add_version_command(subparser):
     parser = add_command(subparser, 'version', 'print the version', cmd_version)
+    parser.add_argument(
+        '--builtin',
+        action='store_true',
+        default=False,
+        help=argparse.SUPPRESS,
+    )
     parser.add_argument(
         '--brief',
         action='store_true',
         default=False,
         help='print the version number only',
     )
-    parser.add_argument(
-        '--build', action='store_true', default=False, help='print the build version'
-    )
 
 
 # entry points
 def cmd_cola(args):
-    from .widgets.main import MainView  # pylint: disable=all
+    from .widgets.main import MainView
 
     status_filter = args.status_filter
     if status_filter:
@@ -480,7 +488,7 @@ def start_cola(context, view):
 
 
 def cmd_about(args):
-    from .widgets import about  # pylint: disable=all
+    from .widgets import about
 
     context = app.application_init(args)
     view = about.about_dialog(context)
@@ -488,7 +496,7 @@ def cmd_about(args):
 
 
 def cmd_am(args):
-    from .widgets.patch import new_apply_patches  # pylint: disable=all
+    from .widgets.patch import new_apply_patches
 
     context = app.application_init(args)
     view = new_apply_patches(context, patches=args.patches)
@@ -496,7 +504,7 @@ def cmd_am(args):
 
 
 def cmd_archive(args):
-    from .widgets import archive  # pylint: disable=all
+    from .widgets import archive
 
     context = app.application_init(args, update=True)
     if args.ref is None:
@@ -506,7 +514,7 @@ def cmd_archive(args):
 
 
 def cmd_branch(args):
-    from .widgets.createbranch import create_new_branch  # pylint: disable=all
+    from .widgets.createbranch import create_new_branch
 
     context = app.application_init(args, update=True)
     view = create_new_branch(context)
@@ -514,7 +522,7 @@ def cmd_branch(args):
 
 
 def cmd_browse(args):
-    from .widgets.browse import worktree_browser  # pylint: disable=all
+    from .widgets.browse import worktree_browser
 
     context = app.application_init(args)
     view = worktree_browser(context, show=False, update=False)
@@ -522,7 +530,7 @@ def cmd_browse(args):
 
 
 def cmd_clone(args):
-    from .widgets import clone  # pylint: disable=all
+    from .widgets import clone
 
     context = app.application_init(args)
     view = clone.clone(context)
@@ -533,7 +541,7 @@ def cmd_clone(args):
 
 
 def cmd_config(args):
-    from .widgets.prefs import preferences  # pylint: disable=all
+    from .widgets.prefs import preferences
 
     context = app.application_init(args)
     view = preferences(context)
@@ -541,7 +549,7 @@ def cmd_config(args):
 
 
 def cmd_dag(args):
-    from .widgets import dag  # pylint: disable=all
+    from .widgets import dag
 
     context = app.application_init(args)
     # cola.main() uses parse_args(), unlike dag.main() which uses
@@ -555,7 +563,7 @@ def cmd_dag(args):
 
 
 def cmd_diff(args):
-    from .difftool import diff_expression  # pylint: disable=all
+    from .difftool import diff_expression
 
     context = app.application_init(args)
     expr = core.list2cmdline(args.args)
@@ -566,7 +574,7 @@ def cmd_diff(args):
 def cmd_fetch(args):
     # TODO: the calls to update_status() can be done asynchronously
     # by hooking into the message_updated notification.
-    from .widgets import remote  # pylint: disable=all
+    from .widgets import remote
 
     context = app.application_init(args)
     context.model.update_status()
@@ -575,7 +583,7 @@ def cmd_fetch(args):
 
 
 def cmd_find(args):
-    from .widgets import finder  # pylint: disable=all
+    from .widgets import finder
 
     context = app.application_init(args)
     paths = core.list2cmdline(args.paths)
@@ -584,7 +592,7 @@ def cmd_find(args):
 
 
 def cmd_grep(args):
-    from .widgets import grep  # pylint: disable=all
+    from .widgets import grep
 
     context = app.application_init(args)
     text = core.list2cmdline(args.args)
@@ -593,7 +601,7 @@ def cmd_grep(args):
 
 
 def cmd_merge(args):
-    from .widgets.merge import Merge  # pylint: disable=all
+    from .widgets.merge import Merge
 
     context = app.application_init(args, update=True)
     view = Merge(context, parent=None, ref=args.ref)
@@ -601,14 +609,14 @@ def cmd_merge(args):
 
 
 def cmd_version(args):
-    from . import version  # pylint: disable=all
+    from . import version
 
-    version.print_version(brief=args.brief, build=args.build)
+    version.print_version(builtin=args.builtin, brief=args.brief)
     return 0
 
 
 def cmd_pull(args):
-    from .widgets import remote  # pylint: disable=all
+    from .widgets import remote
 
     context = app.application_init(args, update=True)
     view = remote.pull(context)
@@ -618,7 +626,7 @@ def cmd_pull(args):
 
 
 def cmd_push(args):
-    from .widgets import remote  # pylint: disable=all
+    from .widgets import remote
 
     context = app.application_init(args, update=True)
     view = remote.push(context)
@@ -655,6 +663,7 @@ def cmd_rebase(args):
         'abort': args.abort,
         'skip': args.skip,
         'edit_todo': args.edit_todo,
+        'update_refs': args.update_refs,
         'upstream': args.upstream,
         'branch': args.branch,
     }
@@ -664,7 +673,7 @@ def cmd_rebase(args):
 
 
 def cmd_recent(args):
-    from .widgets import recent  # pylint: disable=all
+    from .widgets import recent
 
     context = app.application_init(args)
     view = recent.browse_recent_files(context)
@@ -672,7 +681,7 @@ def cmd_recent(args):
 
 
 def cmd_remote(args):
-    from .widgets import editremotes  # pylint: disable=all
+    from .widgets import editremotes
 
     context = app.application_init(args)
     view = editremotes.editor(context, run=False)
@@ -680,7 +689,7 @@ def cmd_remote(args):
 
 
 def cmd_search(args):
-    from .widgets.search import search  # pylint: disable=all
+    from .widgets.search import search
 
     context = app.application_init(args)
     view = search(context)
@@ -688,7 +697,7 @@ def cmd_search(args):
 
 
 def cmd_stash(args):
-    from .widgets import stash  # pylint: disable=all
+    from .widgets import stash
 
     context = app.application_init(args)
     view = stash.view(context, show=False)
@@ -696,9 +705,10 @@ def cmd_stash(args):
 
 
 def cmd_tag(args):
-    from .widgets.createtag import new_create_tag  # pylint: disable=all
+    from .widgets.createtag import new_create_tag
 
     context = app.application_init(args)
+    context.model.update_status()
     view = new_create_tag(context, name=args.name, ref=args.ref, sign=args.sign)
     return app.application_start(context, view)
 
@@ -713,4 +723,4 @@ def shortcut_launch():
     argv = sys.argv[1:]
     if not argv:
         argv = ['cola', '--prompt']
-    return app.winmain(main, argv)
+    return main(argv=argv)
