@@ -1,8 +1,8 @@
 """Provides a selection model to handle selection."""
-from __future__ import absolute_import, division, print_function, unicode_literals
 import collections
 
-from ..observable import Observable
+from qtpy import QtCore
+from qtpy.QtCore import Signal
 
 State = collections.namedtuple('State', 'staged unmerged modified untracked')
 
@@ -12,59 +12,64 @@ def create():
     return SelectionModel()
 
 
-def pick(s):
+def pick(selection):
     """Choose the first list from stage, unmerged, modified, untracked"""
-    if s.staged:
-        files = s.staged
-    elif s.unmerged:
-        files = s.unmerged
-    elif s.modified:
-        files = s.modified
-    elif s.untracked:
-        files = s.untracked
+    if selection.staged:
+        files = selection.staged
+    elif selection.unmerged:
+        files = selection.unmerged
+    elif selection.modified:
+        files = selection.modified
+    elif selection.untracked:
+        files = selection.untracked
     else:
         files = []
     return files
 
 
-def union(s):
+def union(selection):
     """Return the union of all selected items in a sorted list"""
-    return list(sorted(set(s.staged + s.unmerged + s.modified + s.untracked)))
+    values = set(
+        selection.staged + selection.unmerged + selection.modified + selection.untracked
+    )
+    return list(sorted(values))
 
 
-def _filter(a, b):
-    b_set = set(b)
-    a_copy = list(a)
-    last = len(a_copy) - 1
-    for idx, i in enumerate(reversed(a)):
-        if i not in b_set:
-            a.pop(last - idx)
+def _filter(values, remove):
+    """Filter a list in-place by removing items"""
+    remove_set = set(remove)
+    values_copy = list(values)
+    last = len(values_copy) - 1
+    for idx, value in enumerate(reversed(values)):
+        if value not in remove_set:
+            values.pop(last - idx)
 
 
-class SelectionModel(Observable):
+class SelectionModel(QtCore.QObject):
     """Provides information about selected file paths."""
 
-    # Notification message sent out when selection changes
-    message_selection_changed = 'selection_changed'
+    selection_changed = Signal()
 
     # These properties wrap the individual selection items
     # to provide higher-level pseudo-selections.
     unstaged = property(lambda self: self.unmerged + self.modified + self.untracked)
 
     def __init__(self):
-        Observable.__init__(self)
+        super().__init__()
         self.staged = []
         self.unmerged = []
         self.modified = []
         self.untracked = []
         self.line_number = None
 
-    def reset(self):
+    def reset(self, emit=False):
         self.staged = []
         self.unmerged = []
         self.modified = []
         self.untracked = []
         self.line_number = None
+        if emit:
+            self.selection_changed.emit()
 
     def is_empty(self):
         return not (
@@ -77,36 +82,36 @@ class SelectionModel(Observable):
         self.unmerged = s.unmerged
         self.modified = s.modified
         self.untracked = s.untracked
-        self.notify_observers(self.message_selection_changed)
+        self.selection_changed.emit()
 
     def update(self, other):
         _filter(self.staged, other.staged)
         _filter(self.unmerged, other.unmerged)
         _filter(self.modified, other.modified)
         _filter(self.untracked, other.untracked)
-        self.notify_observers(self.message_selection_changed)
+        self.selection_changed.emit()
 
     def selection(self):
         return State(self.staged, self.unmerged, self.modified, self.untracked)
 
     def single_selection(self):
         """Scan across staged, modified, etc. and return a single item."""
-        st = None
-        m = None
-        um = None
-        ut = None
+        staged = None
+        modified = None
+        unmerged = None
+        untracked = None
         if self.staged:
-            st = self.staged[0]
+            staged = self.staged[0]
         elif self.modified:
-            m = self.modified[0]
+            modified = self.modified[0]
         elif self.unmerged:
-            um = self.unmerged[0]
+            unmerged = self.unmerged[0]
         elif self.untracked:
-            ut = self.untracked[0]
-        return State(st, um, m, ut)
+            untracked = self.untracked[0]
+        return State(staged, unmerged, modified, untracked)
 
     def filename(self):
-        paths = [p for p in self.single_selection() if p is not None]
+        paths = [path for path in self.single_selection() if path is not None]
         if paths:
             filename = paths[0]
         else:
