@@ -1,20 +1,17 @@
-# flake8: noqa
-from __future__ import absolute_import, division, print_function, unicode_literals
 import sys
 import re
 from argparse import ArgumentParser
-from functools import partial, reduce
+from functools import partial
 
 from cola import app  # prints a message if Qt cannot be found
-from qtpy import QtCore
 from qtpy import QtGui
 from qtpy import QtWidgets
 from qtpy.QtCore import Qt
 from qtpy.QtCore import Signal
 
-# pylint: disable=ungrouped-imports
 from cola import core
 from cola import difftool
+from cola import gitcmds
 from cola import hotkeys
 from cola import icons
 from cola import qtutils
@@ -64,9 +61,10 @@ def main():
     return view.status
 
 
-def stop(_context, _view):
+def stop(context, _view):
     """All done, cleanup"""
-    QtCore.QThreadPool.globalInstance().waitForDone()
+    context.view.stop()
+    context.runtask.wait()
 
 
 def parse_args():
@@ -94,18 +92,18 @@ class MainWindow(standard.MainWindow):
     """The main git-cola application window"""
 
     def __init__(self, context, parent=None):
-        super(MainWindow, self).__init__(parent)
+        super().__init__(parent)
         self.context = context
         self.status = 1
+        # If the user closes the window without confirmation it's considered cancelled.
+        self.cancelled = True
         self.editor = None
         default_title = '%s - git cola sequence editor' % core.getcwd()
         title = core.getenv('GIT_COLA_SEQ_EDITOR_TITLE', default_title)
         self.setWindowTitle(title)
-
         self.show_help_action = qtutils.add_action(
             self, N_('Show Help'), partial(show_help, context), hotkeys.QUESTION
         )
-
         self.menubar = QtWidgets.QMenuBar(self)
         self.help_menu = self.menubar.addMenu(N_('Help'))
         self.help_menu.addAction(self.show_help_action)
@@ -116,43 +114,48 @@ class MainWindow(standard.MainWindow):
 
     def init_window_size(self):
         """Set the window size on the first initial view"""
-        context = self.context
         if utils.is_darwin():
-            desktop = context.app.desktop()
-            self.resize(desktop.width(), desktop.height())
+            width, height = qtutils.desktop_size()
+            self.resize(width, height)
         else:
             self.showMaximized()
 
     def set_editor(self, editor):
         self.editor = editor
         self.setCentralWidget(editor)
-        editor.exit.connect(self.exit)
+        editor.cancel.connect(self.close)
+        editor.rebase.connect(self.rebase)
         editor.setFocus()
 
     def start(self, _context, _view):
+        """Start background tasks"""
         self.editor.start()
 
-    def exit(self, status):
-        self.status = status
+    def stop(self):
+        """Stop background tasks"""
+        self.editor.stop()
+
+    def rebase(self):
+        """Exit the editor and initiate a rebase"""
+        self.status = self.editor.save()
         self.close()
 
 
 class Editor(QtWidgets.QWidget):
-    exit = Signal(int)
+    cancel = Signal()
+    rebase = Signal()
 
     def __init__(self, context, filename, parent=None):
-        super(Editor, self).__init__(parent)
+        super().__init__(parent)
 
         self.widget_version = 1
-        self.status = 1
         self.context = context
         self.filename = filename
         self.comment_char = comment_char = prefs.comment_char(context)
-        self.cancel_action = core.getenv('GIT_COLA_SEQ_EDITOR_CANCEL_ACTION', 'abort')
 
         self.diff = diff.DiffWidget(context, self)
         self.tree = RebaseTreeWidget(context, comment_char, self)
-        self.filewidget = filelist.FileWidget(context, self)
+        self.filewidget = filelist.FileWidget(context, self, remarks=True)
         self.setFocusProxy(self.tree)
 
         self.rebase_button = qtutils.create_button(
@@ -197,7 +200,11 @@ class Editor(QtWidgets.QWidget):
         self.setLayout(layout)
 
         self.action_rebase = qtutils.add_action(
-            self, N_('Rebase'), self.rebase, hotkeys.CTRL_RETURN, hotkeys.CTRL_ENTER
+            self,
+            N_('Rebase'),
+            self.rebase.emit,
+            hotkeys.CTRL_RETURN,
+            hotkeys.CTRL_ENTER,
         )
 
         self.tree.commits_selected.connect(self.commits_selected)
@@ -206,21 +213,79 @@ class Editor(QtWidgets.QWidget):
         self.tree.external_diff.connect(self.external_diff)
 
         self.filewidget.files_selected.connect(self.diff.files_selected)
+        self.filewidget.remark_toggled.connect(self.remark_toggled_for_files)
 
-        qtutils.connect_button(self.rebase_button, self.rebase)
+        # `git` calls are expensive. When user toggles a remark of all commits touching
+        # selected paths the GUI freezes for a while on a big enough sequence. This
+        # cache is used (commit ID to paths tuple) to minimize calls to git.
+        self.oid_to_paths = {}
+        self.task = None  # A task fills the cache in the background.
+        self.running = False  # This flag stops it.
+
+        qtutils.connect_button(self.rebase_button, self.rebase.emit)
         qtutils.connect_button(self.extdiff_button, self.external_diff)
         qtutils.connect_button(self.help_button, partial(show_help, context))
-        qtutils.connect_button(self.cancel_button, self.cancel)
+        qtutils.connect_button(self.cancel_button, self.cancel.emit)
 
     def start(self):
         insns = core.read(self.filename)
         self.parse_sequencer_instructions(insns)
 
+        # Assume that the tree is filled at this point.
+        self.running = True
+        self.task = qtutils.SimpleTask(self.calculate_oid_to_paths)
+        self.context.runtask.start(self.task)
+
+    def stop(self):
+        self.running = False
+
     # signal callbacks
     def commits_selected(self, commits):
         self.extdiff_button.setEnabled(bool(commits))
 
+    def remark_toggled_for_files(self, remark, filenames):
+        filenames = set(filenames)
+
+        items = self.tree.items()
+        touching_items = []
+
+        for item in items:
+            if not item.is_commit():
+                continue
+            oid = item.oid
+            paths = self.paths_touched_by_oid(oid)
+            if filenames.intersection(paths):
+                touching_items.append(item)
+
+        self.tree.toggle_remark_of_items(remark, touching_items)
+
+    def external_diff(self):
+        items = self.tree.selected_items()
+        if not items:
+            return
+        item = items[0]
+        difftool.diff_expression(self.context, self, item.oid + '^!', hide_expr=True)
+
     # helpers
+
+    def paths_touched_by_oid(self, oid):
+        try:
+            return self.oid_to_paths[oid]
+        except KeyError:
+            pass
+
+        paths = gitcmds.changed_files(self.context, oid)
+        self.oid_to_paths[oid] = paths
+
+        return paths
+
+    def calculate_oid_to_paths(self):
+        """Fills the oid_to_paths cache in the background"""
+        for item in self.tree.items():
+            if not self.running:
+                return
+            self.paths_touched_by_oid(item.oid)
+
     def parse_sequencer_instructions(self, insns):
         idx = 1
         re_comment_char = re.escape(self.comment_char)
@@ -233,9 +298,9 @@ class Editor(QtWidgets.QWidget):
         pick_rgx = re.compile(
             (
                 r'^\s*(%s)?\s*'
-                r'(p|pick|r|reword|e|edit|f|fixup|s|squash)'
-                r'\s+([0-9a-f]{7,40})'
-                r'\s+(.+)$'
+                + r'(p|pick|r|reword|e|edit|f|fixup|s|squash)'
+                + r'\s+([0-9a-f]{7,40})'
+                + r'\s+(.+)$'
             )
             % re_comment_char
         )
@@ -270,64 +335,44 @@ class Editor(QtWidgets.QWidget):
         self.tree.refit()
         self.tree.select_first()
 
-    # actions
-    def cancel(self):
-        if self.cancel_action == 'save':
-            status = self.save('')
-        else:
-            status = 1
-
-        self.status = status
-        self.exit.emit(status)
-
-    def rebase(self):
-        lines = [item.value() for item in self.tree.items()]
-        sequencer_instructions = '\n'.join(lines) + '\n'
-        status = self.save(sequencer_instructions)
-        self.status = status
-        self.exit.emit(status)
-
-    def save(self, string):
+    def save(self, string=None):
         """Save the instruction sheet"""
+
+        if string is None:
+            lines = [item.value() for item in self.tree.items()]
+            # sequencer instructions
+            string = '\n'.join(lines) + '\n'
+
         try:
             core.write(self.filename, string)
             status = 0
-        except (OSError, IOError, ValueError) as exc:
+        except (OSError, ValueError) as exc:
             msg, details = utils.format_exception(exc)
             sys.stderr.write(msg + '\n\n' + details)
             status = 128
         return status
 
-    def external_diff(self):
-        items = self.tree.selected_items()
-        if not items:
-            return
-        item = items[0]
-        difftool.diff_expression(self.context, self, item.oid + '^!', hide_expr=True)
 
-
-# pylint: disable=too-many-ancestors
 class RebaseTreeWidget(standard.DraggableTreeWidget):
     commits_selected = Signal(object)
     external_diff = Signal()
     move_rows = Signal(object, object)
 
     def __init__(self, context, comment_char, parent):
-        super(RebaseTreeWidget, self).__init__(parent=parent)
+        super().__init__(parent=parent)
         self.context = context
         self.comment_char = comment_char
         # header
-        self.setHeaderLabels(
-            [
-                N_('#'),
-                N_('Enabled'),
-                N_('Command'),
-                N_('SHA-1'),
-                N_('Summary'),
-            ]
-        )
+        self.setHeaderLabels([
+            N_('#'),
+            N_('Enabled'),
+            N_('Command'),
+            N_('SHA-1'),
+            N_('Remarks'),
+            N_('Summary'),
+        ])
         self.header().setStretchLastSection(True)
-        self.setColumnCount(5)
+        self.setColumnCount(6)
         self.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
 
         # actions
@@ -351,7 +396,7 @@ class RebaseTreeWidget(standard.DraggableTreeWidget):
             self,
             N_('Reword'),
             lambda: self.set_selected_to(REWORD),
-            *hotkeys.REBASE_REWORD
+            *hotkeys.REBASE_REWORD,
         )
 
         self.action_edit = qtutils.add_action(
@@ -362,14 +407,14 @@ class RebaseTreeWidget(standard.DraggableTreeWidget):
             self,
             N_('Fixup'),
             lambda: self.set_selected_to(FIXUP),
-            *hotkeys.REBASE_FIXUP
+            *hotkeys.REBASE_FIXUP,
         )
 
         self.action_squash = qtutils.add_action(
             self,
             N_('Squash'),
             lambda: self.set_selected_to(SQUASH),
-            *hotkeys.REBASE_SQUASH
+            *hotkeys.REBASE_SQUASH,
         )
 
         self.action_shift_down = qtutils.add_action(
@@ -380,7 +425,16 @@ class RebaseTreeWidget(standard.DraggableTreeWidget):
             self, N_('Shift Up'), self.shift_up, hotkeys.MOVE_UP_TERTIARY
         )
 
-        # pylint: disable=no-member
+        self.toggle_remark_actions = tuple(
+            qtutils.add_action(
+                self,
+                r,
+                lambda remark=r: self.toggle_remark(remark),
+                hotkeys.hotkey(Qt.CTRL | getattr(Qt, 'Key_' + r)),
+            )
+            for r in map(str, range(10))
+        )
+
         self.itemChanged.connect(self.item_changed)
         self.itemSelectionChanged.connect(self.selection_changed)
         self.move_rows.connect(self.move)
@@ -399,6 +453,7 @@ class RebaseTreeWidget(standard.DraggableTreeWidget):
             cmdexec=cmdexec,
             branch=branch,
             comment_char=comment_char,
+            parent=self,
         )
         self.invisibleRootItem().addChild(item)
 
@@ -407,19 +462,17 @@ class RebaseTreeWidget(standard.DraggableTreeWidget):
             item.decorate(self)
 
     def refit(self):
-        self.resizeColumnToContents(0)
-        self.resizeColumnToContents(1)
-        self.resizeColumnToContents(2)
-        self.resizeColumnToContents(3)
-        self.resizeColumnToContents(4)
+        """Resize columns to fit content"""
+        for i in range(RebaseTreeWidgetItem.COLUMN_COUNT - 1):
+            self.resizeColumnToContents(i)
 
-    # actions
     def item_changed(self, item, column):
+        """Validate item ordering when toggling their enabled state"""
         if column == item.ENABLED_COLUMN:
             self.validate()
 
     def validate(self):
-        invalid_first_choice = set([FIXUP, SQUASH])
+        invalid_first_choice = {FIXUP, SQUASH}
         for item in self.items():
             if item.is_enabled() and item.is_commit():
                 if item.command in invalid_first_choice:
@@ -458,10 +511,16 @@ class RebaseTreeWidget(standard.DraggableTreeWidget):
         self.commits_selected.emit(commits)
 
     def toggle_enabled(self):
+        """Toggle the enabled state of each selected item"""
         items = self.selected_items()
-        logic_or = reduce(lambda res, item: res or item.is_enabled(), items, False)
+        enable = should_enable(items, lambda item: item.is_enabled())
         for item in items:
-            item.set_enabled(not logic_or)
+            if enable:
+                needs_update = not item.is_enabled()
+            else:
+                needs_update = item.is_enabled()
+            if needs_update:
+                item.set_enabled(enable)
 
     def select_first(self):
         items = self.items()
@@ -494,6 +553,22 @@ class RebaseTreeWidget(standard.DraggableTreeWidget):
         if idx >= 0:
             self.move_rows.emit(sel_idx, idx)
 
+    def toggle_remark(self, remark):
+        """Toggle remarks for all selected items"""
+        items = self.selected_items()
+        self.toggle_remark_of_items(remark, items)
+
+    def toggle_remark_of_items(self, remark, items):
+        """Toggle remarks for the selected items"""
+        enable = should_enable(items, lambda item: remark in item.remarks)
+        for item in items:
+            needs_update = enable ^ (remark in item.remarks)
+            if needs_update:
+                if enable:
+                    item.add_remark(remark)
+                else:
+                    item.remove_remark(remark)
+
     def move(self, src_idxs, dst_idx):
         moved_items = []
         src_base = sorted(src_idxs)[0]
@@ -521,7 +596,7 @@ class RebaseTreeWidget(standard.DraggableTreeWidget):
     # Qt events
 
     def dropEvent(self, event):
-        super(RebaseTreeWidget, self).dropEvent(event)
+        super().dropEvent(event)
         self.validate()
 
     def contextMenuEvent(self, event):
@@ -536,23 +611,57 @@ class RebaseTreeWidget(standard.DraggableTreeWidget):
         menu.addAction(self.toggle_enabled_action)
         menu.addSeparator()
         menu.addAction(self.copy_oid_action)
-        if len(items) > 1:
-            self.copy_oid_action.setDisabled(True)
+        self.copy_oid_action.setDisabled(len(items) > 1)
         menu.addAction(self.external_diff_action)
-        if len(items) > 1:
-            self.external_diff_action.setDisabled(True)
+        self.external_diff_action.setDisabled(len(items) > 1)
+        menu.addSeparator()
+        menu_toggle_remark = menu.addMenu(N_('Toggle Remark'))
+        for action in self.toggle_remark_actions:
+            menu_toggle_remark.addAction(action)
         menu.exec_(self.mapToGlobal(event.pos()))
 
 
+def should_enable(items, predicate):
+    """Calculate whether items should be toggled on or off.
+
+    If all items are enabled then return False.
+    If all items are disabled then return True.
+    If more items are enabled then return True, otherwise return False.
+    """
+    count = len(items)
+    enabled = sum(predicate(item) for item in items)
+    disabled = len(items) - enabled
+    enable = count > enabled >= disabled or disabled == count
+    return enable
+
+
 class ComboBox(QtWidgets.QComboBox):
     validate = Signal()
 
 
 class RebaseTreeWidgetItem(QtWidgets.QTreeWidgetItem):
+    """A single data row in the rebase tree widget"""
 
+    NUMBER_COLUMN = 0
     ENABLED_COLUMN = 1
     COMMAND_COLUMN = 2
+    COMMIT_COLUMN = 3
+    REMARKS_COLUMN = 4
+    SUMMARY_COLUMN = 5
+    COLUMN_COUNT = 6
     OID_LENGTH = 7
+    COLORS = {
+        '0': ('white', 'darkred'),
+        '1': ('black', 'salmon'),
+        '2': ('black', 'sandybrown'),
+        '3': ('black', 'yellow'),
+        '4': ('black', 'yellowgreen'),
+        '5': ('white', 'forestgreen'),
+        '6': ('white', 'dodgerblue'),
+        '7': ('white', 'royalblue'),
+        '8': ('white', 'slateblue'),
+        '9': ('black', 'rosybrown'),
+    }
 
     def __init__(
         self,
@@ -564,6 +673,7 @@ class RebaseTreeWidgetItem(QtWidgets.QTreeWidgetItem):
         cmdexec='',
         branch='',
         comment_char='#',
+        remarks=(),
         parent=None,
     ):
         QtWidgets.QTreeWidgetItem.__init__(self, parent)
@@ -575,25 +685,30 @@ class RebaseTreeWidgetItem(QtWidgets.QTreeWidgetItem):
         self.cmdexec = cmdexec
         self.branch = branch
         self.comment_char = comment_char
+        self.remarks = remarks
+        self.remarks_label = None
+        self._parent = parent
 
         # if core.abbrev is set to a higher value then we will notice by
         # simply tracking the longest oid we've seen
-        oid_len = self.__class__.OID_LENGTH
+        oid_len = self.OID_LENGTH
         self.__class__.OID_LENGTH = max(len(oid), oid_len)
 
-        self.setText(0, '%02d' % idx)
+        self.setText(self.NUMBER_COLUMN, '%02d' % idx)
         self.set_enabled(enabled)
         # checkbox on 1
         # combo box on 2
         if self.is_exec():
-            self.setText(3, '')
-            self.setText(4, cmdexec)
+            self.setText(self.COMMIT_COLUMN, '')
+            self.setText(self.SUMMARY_COLUMN, cmdexec)
         elif self.is_update_ref():
-            self.setText(3, '')
-            self.setText(4, branch)
+            self.setText(self.COMMIT_COLUMN, '')
+            self.setText(self.SUMMARY_COLUMN, branch)
         else:
-            self.setText(3, oid)
-            self.setText(4, summary)
+            self.setText(self.COMMIT_COLUMN, oid)
+            self.setText(self.SUMMARY_COLUMN, summary)
+
+        self.set_remarks(remarks)
 
         flags = self.flags() | Qt.ItemIsUserCheckable
         flags = flags | Qt.ItemIsDragEnabled
@@ -616,6 +731,7 @@ class RebaseTreeWidgetItem(QtWidgets.QTreeWidgetItem):
             cmdexec=self.cmdexec,
             branch=self.branch,
             comment_char=self.comment_char,
+            remarks=self.remarks,
         )
 
     def decorate(self, parent):
@@ -635,12 +751,15 @@ class RebaseTreeWidgetItem(QtWidgets.QTreeWidgetItem):
         combo.setEnabled(self.is_commit())
 
         signal = combo.currentIndexChanged
-        # pylint: disable=no-member
         signal.connect(lambda x: self.set_command_and_validate(combo))
         combo.validate.connect(parent.validate)
 
         parent.setItemWidget(self, self.COMMAND_COLUMN, combo)
 
+        self.remarks_label = remarks_label = QtWidgets.QLabel()
+        parent.setItemWidget(self, self.REMARKS_COLUMN, remarks_label)
+        self.update_remarks()
+
     def is_exec(self):
         return self.command == EXEC
 
@@ -659,20 +778,55 @@ class RebaseTreeWidgetItem(QtWidgets.QTreeWidgetItem):
         else:
             comment = self.comment_char + ' '
         if self.is_exec():
-            return '%s%s %s' % (comment, self.command, self.cmdexec)
+            return f'{comment}{self.command} {self.cmdexec}'
         if self.is_update_ref():
-            return '%s%s %s' % (comment, self.command, self.branch)
-        return '%s%s %s %s' % (comment, self.command, self.oid, self.summary)
+            return f'{comment}{self.command} {self.branch}'
+        return f'{comment}{self.command} {self.oid} {self.summary}'
 
     def is_enabled(self):
+        """Is the item enabled?"""
         return self.checkState(self.ENABLED_COLUMN) == Qt.Checked
 
     def set_enabled(self, enabled):
+        """Enable the item by checking its enabled checkbox"""
         self.setCheckState(self.ENABLED_COLUMN, enabled and Qt.Checked or Qt.Unchecked)
 
     def toggle_enabled(self):
+        """Toggle the enabled state of the item"""
         self.set_enabled(not self.is_enabled())
 
+    def add_remark(self, remark):
+        """Add a remark to the item"""
+        self.set_remarks(tuple(sorted(set(self.remarks + (remark,)))))
+
+    def remove_remark(self, remark):
+        """Remove a remark from the item"""
+        self.set_remarks(tuple(r for r in self.remarks if r != remark))
+
+    def set_remarks(self, remarks):
+        """Set the remarks and update the remark display"""
+        if remarks == self.remarks:
+            return
+        self.remarks = remarks
+        self.update_remarks()
+        self._parent.resizeColumnToContents(self.REMARKS_COLUMN)
+
+    def update_remarks(self):
+        """Update the remarks label display to match the current remarks"""
+        label = self.remarks_label
+        if label is None:
+            return
+        label_text = ''
+        for remark in self.remarks:
+            fg_color, bg_color = self.COLORS[remark]
+            label_text += f"""
+                <span style="
+                    color: {fg_color};
+                    background-color: {bg_color};
+                ">&nbsp;{remark} </span>
+            """
+        label.setText(label_text)
+
     def set_command(self, command):
         """Set the item to a different command, no-op for exec items"""
         if self.is_exec():
@@ -691,6 +845,7 @@ class RebaseTreeWidgetItem(QtWidgets.QTreeWidgetItem):
         self.refresh()
 
     def set_command_and_validate(self, combo):
+        """Set the command and validate the command order"""
         command = COMMANDS[combo.currentIndex()]
         self.set_command(command)
         self.combo.validate.emit()
