@@ -1,8 +1,11 @@
 """Provide git-cola's version number"""
-from __future__ import absolute_import, division, print_function, unicode_literals
 import os
 import sys
 
+from .git import STDOUT
+from .decorators import memoize
+from ._version import VERSION
+
 try:
     if sys.version_info < (3, 8):
         import importlib_metadata as metadata
@@ -16,9 +19,6 @@ if __name__ == '__main__':
     srcdir = os.path.dirname(os.path.dirname(__file__))
     sys.path.insert(1, srcdir)
 
-from .git import STDOUT  # noqa
-from .decorators import memoize  # noqa
-from ._version import VERSION  # noqa
 
 # minimum version requirements
 _versions = {
@@ -39,14 +39,20 @@ _versions = {
     'force-with-lease': '1.8.5',
     # git submodule update --recursive was introduced in 1.6.5
     'submodule-update-recursive': '1.6.5',
-    # git include.path pseudo-variable was introduced in 1.7.10
+    # git include.path pseudo-variable was introduced in 1.7.10.
     'config-includes': '1.7.10',
+    # git config --show-scope was introduced in 2.26.0
+    'config-show-scope': '2.26.0',
+    # git config --show-origin was introduced in 2.8.0
+    'config-show-origin': '2.8.0',
     # git for-each-ref --sort=version:refname
     'version-sort': '2.7.0',
     # Qt support for QT_AUTO_SCREEN_SCALE_FACTOR and QT_SCALE_FACTOR
     'qt-hidpi-scale': '5.6.0',
     # git rev-parse --show-superproject-working-tree was added in 2.13.0
     'show-superproject-working-tree': '2.13.0',
+    # git rebase --update-refs was added in 2.38.0
+    'rebase-update-refs': '2.38.0',
 }
 
 
@@ -66,6 +72,11 @@ def version():
     return pkg_version
 
 
+def builtin_version():
+    """Returns the version recorded in cola/_version.py"""
+    return VERSION
+
+
 @memoize
 def check_version(min_ver, ver):
     """Check whether ver is greater or equal to min_ver"""
@@ -88,12 +99,12 @@ def check_git(context, key):
 def version_to_list(value):
     """Convert a version string to a list of numbers or strings"""
     ver_list = []
-    for p in value.split('.'):
+    for part in value.split('.'):
         try:
-            n = int(p)
+            number = int(part)
         except ValueError:
-            n = p
-        ver_list.append(n)
+            number = part
+        ver_list.append(number)
     return ver_list
 
 
@@ -117,12 +128,15 @@ def git_version(context):
 
 
 def cola_version():
+    """A version string for consumption by humans"""
     suffix = version()
     return 'cola version %s' % suffix
 
 
-def print_version(brief=False):
-    if brief:
+def print_version(builtin=False, brief=False):
+    if builtin:
+        msg = builtin_version()
+    elif brief:
         msg = version()
     else:
         msg = cola_version()
