@@ -1,5 +1,3 @@
-# encoding: utf-8
-from __future__ import division, absolute_import, unicode_literals
 import platform
 import webbrowser
 import sys
@@ -32,19 +30,18 @@ class ExpandingTabBar(QtWidgets.QTabBar):
     The setExpanding(True) method does not work in practice because
     it respects the OS style.  We override the style by implementing
     tabSizeHint() so that we can specify the size explicitly.
-
     """
 
     def tabSizeHint(self, tab_index):
-        width = self.parent().width() / max(1, self.count()) - 1
-        size = super(ExpandingTabBar, self).tabSizeHint(tab_index)
+        width = self.parent().width() // max(2, self.count()) - 1
+        size = super().tabSizeHint(tab_index)
         size.setWidth(width)
         return size
 
 
 class ExpandingTabWidget(QtWidgets.QTabWidget):
     def __init__(self, parent=None):
-        super(ExpandingTabWidget, self).__init__(parent)
+        super().__init__(parent)
         self.setTabBar(ExpandingTabBar(self))
 
     def resizeEvent(self, event):
@@ -54,7 +51,7 @@ class ExpandingTabWidget(QtWidgets.QTabWidget):
         width = event.size().width()
         height = self.tabBar().height()
         self.tabBar().resize(width, height)
-        return super(ExpandingTabWidget, self).resizeEvent(event)
+        return super().resizeEvent(event)
 
 
 class AboutView(QtWidgets.QDialog):
@@ -126,7 +123,7 @@ def copyright_text():
     return """
 Git Cola: The highly caffeinated Git GUI
 
-Copyright (C) 2007-2020 David Aguilar and contributors
+Copyright (C) 2007-2024 David Aguilar and contributors
 
 This program is free software: you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -149,13 +146,16 @@ If not, see http://www.gnu.org/licenses/.
 def version_text(context):
     git_version = version.git_version(context)
     cola_version = version.version()
-    build_version = version.build_version()
     python_path = sys.executable
     python_version = sys.version
     qt_version = qtpy.QT_VERSION
     qtpy_version = qtpy.__version__
     pyqt_api_name = qtpy.API_NAME
-    if qtpy.PYQT5 or qtpy.PYQT4:
+    if (
+        getattr(qtpy, 'PYQT6', False)
+        or getattr(qtpy, 'PYQT5', False)
+        or getattr(qtpy, 'PYQT4', False)
+    ):
         pyqt_api_version = qtpy.PYQT_VERSION
     elif qtpy.PYSIDE:
         pyqt_api_version = qtpy.PYSIDE_VERSION
@@ -164,15 +164,8 @@ def version_text(context):
 
     platform_version = platform.platform()
 
-    # Only show the build version if _build_version.py exists
-    if build_version:
-        build_version = '(%s)' % build_version
-    else:
-        build_version = ''
-
     scope = dict(
         cola_version=cola_version,
-        build_version=build_version,
         git_version=git_version,
         platform_version=platform_version,
         pyqt_api_name=pyqt_api_name,
@@ -187,7 +180,7 @@ def version_text(context):
         N_(
             """
         <br>
-            Git Cola version %(cola_version)s %(build_version)s
+            Git Cola version %(cola_version)s
         <ul>
             <li> %(platform_version)s
             <li> Python (%(python_path)s) %(python_version)s
@@ -202,31 +195,12 @@ def version_text(context):
     )
 
 
-def link(url, text, palette=None):
-    if palette is None:
-        palette = QtGui.QPalette()
-
-    color = palette.color(QtGui.QPalette.Foreground)
-    rgb = 'rgb(%s, %s, %s)' % (color.red(), color.green(), color.blue())
-    scope = dict(rgb=rgb, text=text, url=url)
-
-    return (
-        """
-        <a style="font-style: italic; text-decoration: none; color: %(rgb)s;"
-            href="%(url)s">
-            %(text)s
-        </a>
-    """
-        % scope
-    )
-
-
 def mailto(email, text, palette):
-    return link('mailto:%s' % email, text, palette) + '<br>'
+    return qtutils.link('mailto:%s' % email, text, palette) + '<br>'
 
 
 def render_authors(authors):
-    """Render a list of author details into richtext html"""
+    """Render a list of author details into rich text html"""
     for x in authors:
         x.setdefault('email', '')
 
@@ -270,7 +244,7 @@ def authors_text():
         # Please submit a pull request if you would like to include your
         # email address in the about screen.
         # See the `generate-about` script in the "todo" branch.
-        # vim :read! ./Meta/generate-about
+        # vim :read! ./todo/generate-about
         dict(
             name='David Aguilar',
             title=N_('Maintainer (since 2007) and developer'),
@@ -388,7 +362,7 @@ def authors_text():
         dict(name='Markus Heidelberg', title=N_('Developer')),
     )
     bug_url = 'https://github.com/git-cola/git-cola/issues'
-    bug_link = link(bug_url, bug_url)
+    bug_link = qtutils.link(bug_url, bug_url)
     scope = dict(bug_link=bug_link)
     prelude = (
         N_(
@@ -455,7 +429,7 @@ def translators_text():
     )
 
     bug_url = 'https://github.com/git-cola/git-cola/issues'
-    bug_link = link(bug_url, bug_url)
+    bug_link = qtutils.link(bug_url, bug_url)
     scope = dict(bug_link=bug_link)
 
     prelude = (
@@ -493,9 +467,9 @@ def translators_text():
 def show_shortcuts():
     hotkeys_html = resources.doc(N_('hotkeys.html'))
     try:
-        from qtpy import QtWebEngineWidgets  # pylint: disable=all
+        from qtpy import QtWebEngineWidgets
     except (ImportError, qtpy.PythonQtError):
-        # redhat disabled QtWebKit in their qt build but don't punish the users
+        # Redhat disabled QtWebKit in their Qt build but don't punish the users
         webbrowser.open_new_tab('file://' + hotkeys_html)
         return
 
