@@ -1,4 +1,3 @@
-from __future__ import division, absolute_import, unicode_literals
 from functools import partial
 
 from qtpy import QtCore
@@ -15,6 +14,7 @@ from .. import hotkeys
 from .. import icons
 from .. import textwrap
 from .. import qtutils
+from .. import spellcheck
 from ..interaction import Interaction
 from ..gitcmds import commit_message_path
 from ..i18n import N_
@@ -23,23 +23,26 @@ from ..models import prefs
 from ..qtutils import get
 from ..utils import Group
 from . import defs
+from . import standard
 from .selectcommits import select_commits
-from .spellcheck import SpellCheckTextEdit
-from .text import HintedLineEdit
+from .spellcheck import SpellCheckLineEdit, SpellCheckTextEdit
+from .text import anchor_mode
 
 
 class CommitMessageEditor(QtWidgets.QFrame):
-    commit_message_changed = Signal(object)
+    commit_finished = Signal(object)
     cursor_changed = Signal(int, int)
     down = Signal()
     up = Signal()
-    updated = Signal()
 
     def __init__(self, context, parent):
         QtWidgets.QFrame.__init__(self, parent)
+        cfg = context.cfg
         self.context = context
         self.model = model = context.model
         self.spellcheck_initialized = False
+        self.spellcheck = spellcheck.NorvigSpellCheck()
+        self.spellcheck.set_dictionary(cfg.get('cola.dictionary', None))
 
         self._linebreak = None
         self._textwidth = None
@@ -61,43 +64,43 @@ class CommitMessageEditor(QtWidgets.QFrame):
 
         self.launch_editor = actions.launch_editor_at_line(context, self)
         self.launch_difftool = actions.launch_difftool(context, self)
-        self.stage_or_unstage = actions.stage_or_unstage(context, self)
 
         self.move_up = actions.move_up(self)
         self.move_down = actions.move_down(self)
 
-        # Menu acctions
+        # Menu actions
         self.menu_actions = menu_actions = [
-            None,
             self.signoff_action,
             self.commit_action,
             None,
             self.launch_editor,
             self.launch_difftool,
-            self.stage_or_unstage,
             None,
             self.move_up,
             self.move_down,
+            None,
         ]
 
         # Widgets
-        self.summary = CommitSummaryLineEdit(context)
+        self.summary = CommitSummaryLineEdit(context, check=self.spellcheck)
         self.summary.setMinimumHeight(defs.tool_button_height)
         self.summary.menu_actions.extend(menu_actions)
 
-        cfg = context.cfg
-        self.summary_validator = MessageValidator(context, parent=self.summary)
-        self.summary.setValidator(self.summary_validator)
-
-        self.description = CommitMessageTextEdit(context, parent=self)
-        self.description.set_dictionary(cfg.get('cola.dictionary', None))
+        self.description = CommitMessageTextEdit(
+            context, check=self.spellcheck, parent=self
+        )
         self.description.menu_actions.extend(menu_actions)
 
-        commit_button_tooltip = N_('Commit staged changes\n' 'Shortcut: Ctrl+Enter')
+        commit_button_tooltip = N_('Commit staged changes\nShortcut: Ctrl+Enter')
         self.commit_button = qtutils.create_button(
             text=N_('Commit@@verb'), tooltip=commit_button_tooltip, icon=icons.commit()
         )
         self.commit_group = Group(self.commit_action, self.commit_button)
+        self.commit_progress_bar = standard.progress_bar(
+            self,
+            disable=(self.commit_button, self.summary, self.description),
+        )
+        self.commit_progress_bar.setMaximumHeight(defs.small_icon)
 
         self.actions_menu = qtutils.create_menu(N_('Actions'), self)
         self.actions_button = qtutils.create_toolbutton(
@@ -132,9 +135,9 @@ class CommitMessageEditor(QtWidgets.QFrame):
         # Spell checker
         self.check_spelling_action = self.actions_menu.addAction(N_('Check Spelling'))
         self.check_spelling_action.setCheckable(True)
-        spellcheck = prefs.spellcheck(context)
-        self.check_spelling_action.setChecked(spellcheck)
-        self.toggle_check_spelling(spellcheck)
+        spell_check = prefs.spellcheck(context)
+        self.check_spelling_action.setChecked(spell_check)
+        self.toggle_check_spelling(spell_check)
 
         # Line wrapping
         self.autowrap_action = self.actions_menu.addAction(N_('Auto-Wrap Lines'))
@@ -177,39 +180,25 @@ class CommitMessageEditor(QtWidgets.QFrame):
             self.check_spelling_action, self.toggle_check_spelling
         )
 
-        # Handle the one-off autowrapping
+        # Handle the one-off auto-wrapping
         qtutils.connect_action_bool(self.autowrap_action, self.set_linebreak)
 
-        qtutils.add_action(
-            self.summary, N_('Move Down'), self.focus_description, *hotkeys.ACCEPT
-        )
+        self.summary.accepted.connect(self.focus_description)
+        self.summary.down_pressed.connect(self.summary_cursor_down)
 
-        qtutils.add_action(
-            self.summary, N_('Move Down'), self.summary_cursor_down, hotkeys.DOWN
-        )
-
-        self.selection = selection = context.selection
-        selection.add_observer(selection.message_selection_changed, self.updated.emit)
-
-        self.model.add_observer(
-            self.model.message_commit_message_changed, self.commit_message_changed.emit
-        )
-
-        self.commit_message_changed.connect(
+        self.model.commit_message_changed.connect(
             self.set_commit_message, type=Qt.QueuedConnection
         )
+        self.commit_finished.connect(self._commit_finished, type=Qt.QueuedConnection)
 
         self.summary.cursor_changed.connect(self.cursor_changed.emit)
         self.description.cursor_changed.connect(
             # description starts at line 2
             lambda row, col: self.cursor_changed.emit(row + 2, col)
         )
-
-        # pylint: disable=no-member
         self.summary.textChanged.connect(self.commit_summary_changed)
         self.description.textChanged.connect(self._commit_message_changed)
         self.description.leave.connect(self.focus_summary)
-        self.updated.connect(self.refresh)
 
         self.commit_group.setEnabled(False)
 
@@ -230,7 +219,7 @@ class CommitMessageEditor(QtWidgets.QFrame):
         self.setFont(qtutils.diff_font(context))
         self.setFocusProxy(self.summary)
 
-        cfg.add_observer(cfg.message_user_config_changed, self.config_changed)
+        cfg.user_config_changed.connect(self.config_changed)
 
     def config_changed(self, key, value):
         if key != prefs.SPELL_CHECK:
@@ -240,21 +229,12 @@ class CommitMessageEditor(QtWidgets.QFrame):
         self.check_spelling_action.setChecked(value)
         self.toggle_check_spelling(value)
 
-    def refresh(self):
-        enabled = self.model.stageable() or self.model.unstageable()
-        if self.model.stageable():
-            text = N_('Stage')
-        else:
-            text = N_('Unstage')
-        self.stage_or_unstage.setEnabled(enabled)
-        self.stage_or_unstage.setText(text)
-
     def set_initial_size(self):
         self.setMaximumHeight(133)
         QtCore.QTimer.singleShot(1, self.restore_size)
 
     def restore_size(self):
-        self.setMaximumHeight(2 ** 13)
+        self.setMaximumHeight(2**13)
 
     def focus_summary(self):
         self.summary.setFocus()
@@ -270,15 +250,10 @@ class CommitMessageEditor(QtWidgets.QFrame):
         subsequence "down" press moves to the end of the line.
 
         """
-        cur_position = self.summary.cursorPosition()
-        end_position = len(get(self.summary))
-        if cur_position == end_position:
-            self.focus_description()
-        else:
-            self.summary.setCursorPosition(end_position)
+        self.focus_description()
 
     def commit_message(self, raw=True):
-        """Return the commit message as a unicode string"""
+        """Return the commit message as a Unicode string"""
         summary = get(self.summary)
         if raw:
             description = get(self.description)
@@ -298,7 +273,7 @@ class CommitMessageEditor(QtWidgets.QFrame):
             return text
         return textwrap.word_wrap(text, self._tabwidth, self._textwidth)
 
-    def commit_summary_changed(self, value):
+    def commit_summary_changed(self):
         """Respond to changes to the `summary` field
 
         Newlines can enter the `summary` field when pasting, which is
@@ -307,6 +282,7 @@ class CommitMessageEditor(QtWidgets.QFrame):
         "extended description" field.
 
         """
+        value = self.summary.value()
         if '\n' in value:
             summary, description = value.split('\n', 1)
             description = description.lstrip('\n')
@@ -415,9 +391,8 @@ class CommitMessageEditor(QtWidgets.QFrame):
     def set_linebreak(self, brk):
         self._linebreak = brk
         self.description.set_linebreak(brk)
-        blocksignals = self.autowrap_action.blockSignals(True)
-        self.autowrap_action.setChecked(brk)
-        self.autowrap_action.blockSignals(blocksignals)
+        with qtutils.BlockSignals(self.autowrap_action):
+            self.autowrap_action.setChecked(brk)
 
     def setFont(self, font):
         """Pass the setFont() calls down to the text widgets"""
@@ -427,10 +402,9 @@ class CommitMessageEditor(QtWidgets.QFrame):
     def set_mode(self, mode):
         can_amend = not self.model.is_merging
         checked = mode == self.model.mode_amend
-        blocksignals = self.amend_action.blockSignals(True)
-        self.amend_action.setEnabled(can_amend)
-        self.amend_action.setChecked(checked)
-        self.amend_action.blockSignals(blocksignals)
+        with qtutils.BlockSignals(self.amend_action):
+            self.amend_action.setEnabled(can_amend)
+            self.amend_action.setChecked(checked)
 
     def commit(self):
         """Attempt to create a commit from the index and commit message."""
@@ -460,7 +434,7 @@ class CommitMessageEditor(QtWidgets.QFrame):
             )
             if self.model.modified:
                 informative_text = N_(
-                    'Would you like to stage and ' 'commit all modified files?'
+                    'Would you like to stage and commit all modified files?'
                 )
                 if not Interaction.confirm(
                     N_('Stage and commit?'),
@@ -488,7 +462,7 @@ class CommitMessageEditor(QtWidgets.QFrame):
                 N_(
                     'This commit has already been published.\n'
                     'This operation will rewrite published history.\n'
-                    'You probably don\'t want to do this.'
+                    "You probably don't want to do this."
                 ),
                 N_('Amend the published commit?'),
                 N_('Amend Commit'),
@@ -499,9 +473,24 @@ class CommitMessageEditor(QtWidgets.QFrame):
             return
         no_verify = get(self.bypass_commit_hooks_action)
         sign = get(self.sign_action)
-        cmds.do(cmds.Commit, context, amend, msg, sign, no_verify=no_verify)
         self.bypass_commit_hooks_action.setChecked(False)
 
+        task = qtutils.SimpleTask(
+            cmds.run(cmds.Commit, context, amend, msg, sign, no_verify=no_verify)
+        )
+        self.context.runtask.start(
+            task,
+            finish=self.commit_finished.emit,
+            progress=self.commit_progress_bar,
+        )
+
+    def _commit_finished(self, task):
+        """Reset widget state on completion of the commit task"""
+        title = N_('Commit failed')
+        status, out, err = task.result
+        Interaction.command(title, 'git commit', status, out, err)
+        self.setFocus()
+
     def build_fixup_menu(self):
         self.build_commits_menu(
             cmds.LoadFixupMessage,
@@ -523,14 +512,14 @@ class CommitMessageEditor(QtWidgets.QFrame):
         commits = dag.RepoReader(context, params)
 
         menu_commits = []
-        for idx, c in enumerate(commits.get()):
-            menu_commits.insert(0, c)
+        for idx, commit in enumerate(commits.get()):
+            menu_commits.insert(0, commit)
             if idx > 5:
                 continue
 
         menu.clear()
-        for c in menu_commits:
-            menu.addAction(prefix + c.summary, cmds.run(cmd, context, c.oid))
+        for commit in menu_commits:
+            menu.addAction(prefix + commit.summary, cmds.run(cmd, context, commit.oid))
 
         if len(commits) == 6:
             menu.addSeparator()
@@ -554,10 +543,10 @@ class CommitMessageEditor(QtWidgets.QFrame):
         self.choose_commit(cmds.LoadFixupMessage)
 
     def toggle_check_spelling(self, enabled):
-        spellcheck = self.description.spellcheck
+        spell_check = self.spellcheck
         cfg = self.context.cfg
 
-        if cfg.get_user(prefs.SPELL_CHECK) != enabled:
+        if prefs.spellcheck(self.context) != enabled:
             cfg.set_user(prefs.SPELL_CHECK, enabled)
         if enabled and not self.spellcheck_initialized:
             # Add our name to the dictionary
@@ -565,101 +554,86 @@ class CommitMessageEditor(QtWidgets.QFrame):
             user_name = cfg.get('user.name')
             if user_name:
                 for part in user_name.split():
-                    spellcheck.add_word(part)
+                    spell_check.add_word(part)
 
             # Add our email address to the dictionary
             user_email = cfg.get('user.email')
             if user_email:
                 for part in user_email.split('@'):
                     for elt in part.split('.'):
-                        spellcheck.add_word(elt)
+                        spell_check.add_word(elt)
 
             # git jargon
-            spellcheck.add_word('Acked')
-            spellcheck.add_word('Signed')
-            spellcheck.add_word('Closes')
-            spellcheck.add_word('Fixes')
+            spell_check.add_word('Acked')
+            spell_check.add_word('Signed')
+            spell_check.add_word('Closes')
+            spell_check.add_word('Fixes')
 
+        self.summary.highlighter.enable(enabled)
         self.description.highlighter.enable(enabled)
 
 
-class MessageValidator(QtGui.QValidator):
-    """Prevent invalid branch names"""
+class CommitSummaryLineEdit(SpellCheckLineEdit):
+    """Text input field for the commit summary"""
 
-    config_updated = Signal()
+    down_pressed = Signal()
+    accepted = Signal()
 
-    def __init__(self, context, parent=None):
-        super(MessageValidator, self).__init__(parent)
-        self.context = context
+    def __init__(self, context, check=None, parent=None):
+        hint = N_('Commit summary')
+        SpellCheckLineEdit.__init__(self, context, hint, check=check, parent=parent)
         self._comment_char = None
-        self._cfg = cfg = context.cfg
-        self.refresh()
-        # pylint: disable=no-member
-        self.config_updated.connect(self.refresh, type=Qt.QueuedConnection)
-        cfg.add_observer(cfg.message_updated, self.emit_config_updated)
-        self.destroyed.connect(self.teardown)
+        self._refresh_config()
 
-    def teardown(self):
-        self._cfg.remove_observer(self.emit_config_updated)
+        self.textChanged.connect(self._update_summary_text, Qt.QueuedConnection)
+        context.cfg.updated.connect(self._refresh_config, type=Qt.QueuedConnection)
 
-    def emit_config_updated(self):
-        self.config_updated.emit()
-
-    def refresh(self):
+    def _refresh_config(self):
         """Update comment char in response to config changes"""
         self._comment_char = prefs.comment_char(self.context)
 
-    def validate(self, string, idx):
-        """Scrub whitespace and validate the commit message"""
-        string = string.lstrip()
-        if string.startswith(self._comment_char):
-            state = self.Invalid
-        else:
-            state = self.Acceptable
-        return (state, string, idx)
+    def _update_summary_text(self):
+        """Prevent commit messages from starting with comment characters"""
+        value = self.value()
+        if self._comment_char and value.startswith(self._comment_char):
+            cursor = self.textCursor()
+            position = cursor.position()
+
+            value = value.lstrip()
+            if self._comment_char:
+                value = value.lstrip(self._comment_char).lstrip()
+
+            self.set_value(value, block=True)
+
+            value = self.value()
+            if position > 1:
+                position = max(0, min(position - 1, len(value) - 1))
+                cursor.setPosition(position)
+                self.setTextCursor(cursor)
+
+    def keyPressEvent(self, event):
+        """Allow "Enter" to focus into the extended description field"""
+        event_key = event.key()
+        if event_key in (
+            Qt.Key_Enter,
+            Qt.Key_Return,
+        ):
+            self.accepted.emit()
+            return
+        SpellCheckLineEdit.keyPressEvent(self, event)
 
 
-class CommitSummaryLineEdit(HintedLineEdit):
-
-    cursor = Signal(int, int)
-
-    def __init__(self, context, parent=None):
-        hint = N_('Commit summary')
-        HintedLineEdit.__init__(self, context, hint, parent=parent)
-        self.menu_actions = []
-
-    def build_menu(self):
-        menu = self.createStandardContextMenu()
-        add_menu_actions(menu, self.menu_actions)
-        return menu
-
-    def contextMenuEvent(self, event):
-        menu = self.build_menu()
-        menu.exec_(self.mapToGlobal(event.pos()))
-
-
-# pylint: disable=too-many-ancestors
 class CommitMessageTextEdit(SpellCheckTextEdit):
     leave = Signal()
 
-    def __init__(self, context, parent=None):
+    def __init__(self, context, check=None, parent=None):
         hint = N_('Extended description...')
-        SpellCheckTextEdit.__init__(self, context, hint, parent)
-        self.menu_actions = []
+        SpellCheckTextEdit.__init__(self, context, hint, check=check, parent=parent)
 
         self.action_emit_leave = qtutils.add_action(
             self, 'Shift Tab', self.leave.emit, hotkeys.LEAVE
         )
 
-    def build_menu(self):
-        menu, _ = self.context_menu()
-        add_menu_actions(menu, self.menu_actions)
-        return menu
-
-    def contextMenuEvent(self, event):
-        menu = self.build_menu()
-        menu.exec_(self.mapToGlobal(event.pos()))
-
     def keyPressEvent(self, event):
         if event.key() == Qt.Key_Up:
             cursor = self.textCursor()
@@ -670,8 +644,7 @@ class CommitMessageTextEdit(SpellCheckTextEdit):
                 # Otherwise, emit a signal so that the parent can
                 # change focus.
                 if cursor.hasSelection():
-                    cursor.setPosition(0)
-                    self.setTextCursor(cursor)
+                    self.set_cursor_position(0)
                 else:
                     self.leave.emit()
                 event.accept()
@@ -697,10 +670,8 @@ class CommitMessageTextEdit(SpellCheckTextEdit):
             text_after = all_text[position:]
             lines_after = text_after.count('\n')
             if lines_after == 0:
-                if event.modifiers() & Qt.ShiftModifier:
-                    mode = QtGui.QTextCursor.KeepAnchor
-                else:
-                    mode = QtGui.QTextCursor.MoveAnchor
+                select = event.modifiers() & Qt.ShiftModifier
+                mode = anchor_mode(select)
                 cursor.setPosition(len(all_text), mode)
                 self.setTextCursor(cursor)
                 event.accept()
@@ -709,14 +680,5 @@ class CommitMessageTextEdit(SpellCheckTextEdit):
 
     def setFont(self, font):
         SpellCheckTextEdit.setFont(self, font)
-        fm = self.fontMetrics()
-        self.setMinimumSize(QtCore.QSize(1, fm.height() * 2))
-
-
-def add_menu_actions(menu, menu_actions):
-    """Add actions to a menu, treating None as a separator"""
-    for action in menu_actions:
-        if action is None:
-            menu.addSeparator()
-        else:
-            menu.addAction(action)
+        width, height = qtutils.text_size(font, 'MMMM')
+        self.setMinimumSize(QtCore.QSize(width, height * 2))
