@@ -1,4 +1,3 @@
-from __future__ import absolute_import, division, print_function, unicode_literals
 from functools import partial
 import os
 import re
@@ -27,8 +26,11 @@ from .. import utils
 from .. import qtutils
 from .text import TextDecorator
 from .text import VimHintedPlainTextEdit
+from .text import PlainTextLabel
+from .text import RichTextLabel
 from .text import TextSearchWidget
 from . import defs
+from . import standard
 from . import imageview
 
 
@@ -183,7 +185,6 @@ class DiffSyntaxHighlighter(QtGui.QSyntaxHighlighter):
         return state, formats
 
 
-# pylint: disable=too-many-ancestors
 class DiffTextEdit(VimHintedPlainTextEdit):
     """A textedit for interacting with diff text"""
 
@@ -211,17 +212,14 @@ class DiffTextEdit(VimHintedPlainTextEdit):
         self.copy_diff_action.setIcon(icons.copy())
         self.copy_diff_action.setEnabled(False)
         self.menu_actions.append(self.copy_diff_action)
-
-        # pylint: disable=no-member
         self.cursorPositionChanged.connect(self._cursor_changed)
         self.selectionChanged.connect(self._selection_changed)
 
     def setFont(self, font):
         """Override setFont() so that we can use a custom "block" cursor"""
-        super(DiffTextEdit, self).setFont(font)
+        super().setFont(font)
         if prefs.block_cursor(self.context):
-            metrics = QtGui.QFontMetrics(font)
-            width = metrics.width('M')
+            width = qtutils.text_width(font, 'M')
             self.setCursorWidth(width)
 
     def _cursor_changed(self):
@@ -236,7 +234,7 @@ class DiffTextEdit(VimHintedPlainTextEdit):
         self.copy_diff_action.setEnabled(selected)
 
     def resizeEvent(self, event):
-        super(DiffTextEdit, self).resizeEvent(event)
+        super().resizeEvent(event)
         if self.numbers:
             self.numbers.refresh_size()
 
@@ -349,8 +347,9 @@ class DiffLineNumbers(TextDecorator):
         self.parser = diffparse.DiffLines()
         self.formatter = diffparse.FormatDigits()
 
-        self.setFont(qtutils.diff_font(context))
-        self._char_width = self.fontMetrics().width('0')
+        font = qtutils.diff_font(context)
+        self.setFont(font)
+        self._char_width = qtutils.text_width(font, 'M')
 
         QPalette = QtGui.QPalette
         self._palette = palette = self.palette()
@@ -471,18 +470,28 @@ class Viewer(QtWidgets.QFrame):
     INDEX_IMAGE = 1
 
     def __init__(self, context, parent=None):
-        super(Viewer, self).__init__(parent)
+        super().__init__(parent)
 
         self.context = context
         self.model = model = context.model
         self.images = []
         self.pixmaps = []
         self.options = options = Options(self)
+        self.filename = PlainTextLabel(parent=self)
+        self.filename.setAlignment(Qt.AlignVCenter | Qt.AlignLeft)
+        font = self.font()
+        font.setItalic(True)
+        self.filename.setFont(font)
+        self.filename.elide()
         self.text = DiffEditor(context, options, self)
         self.image = imageview.ImageView(parent=self)
         self.image.setFocusPolicy(Qt.NoFocus)
         self.search_widget = TextSearchWidget(self.text, self)
         self.search_widget.hide()
+        self._drag_has_patches = False
+
+        self.setAcceptDrops(True)
+        self.setFocusProxy(self.text)
 
         stack = self.stack = QtWidgets.QStackedWidget(self)
         stack.addWidget(self.text)
@@ -509,8 +518,6 @@ class Viewer(QtWidgets.QFrame):
         options.image_mode.currentIndexChanged.connect(lambda _: self.render())
         options.zoom_mode.currentIndexChanged.connect(lambda _: self.render())
 
-        self.setFocusProxy(self.text)
-
         self.search_action = qtutils.add_action(
             self,
             N_('Search in Diff'),
@@ -518,6 +525,38 @@ class Viewer(QtWidgets.QFrame):
             hotkeys.SEARCH,
         )
 
+    def dragEnterEvent(self, event):
+        """Accepts drops if the mimedata contains patches"""
+        super().dragEnterEvent(event)
+        patches = get_patches_from_mimedata(event.mimeData())
+        if patches:
+            event.acceptProposedAction()
+            self._drag_has_patches = True
+
+    def dragLeaveEvent(self, event):
+        """End the drag+drop interaction"""
+        super().dragLeaveEvent(event)
+        if self._drag_has_patches:
+            event.accept()
+        else:
+            event.ignore()
+        self._drag_has_patches = False
+
+    def dropEvent(self, event):
+        """Apply patches when dropped onto the widget"""
+        if not self._drag_has_patches:
+            event.ignore()
+            return
+        event.setDropAction(Qt.CopyAction)
+        super().dropEvent(event)
+        self._drag_has_patches = False
+
+        patches = get_patches_from_mimedata(event.mimeData())
+        if patches:
+            apply_patches(self.context, patches=patches)
+
+        event.accept()  # must be called after dropEvent()
+
     def show_search_diff(self):
         """Show a dialog for searching diffs"""
         # The diff search is only active in text mode.
@@ -525,10 +564,11 @@ class Viewer(QtWidgets.QFrame):
             return
         if not self.search_widget.isVisible():
             self.search_widget.show()
-        self.search_widget.setFocus(True)
+        self.search_widget.setFocus()
 
     def export_state(self, state):
         state['show_diff_line_numbers'] = self.options.show_line_numbers.isChecked()
+        state['show_diff_filenames'] = self.options.show_filenames.isChecked()
         state['image_diff_mode'] = self.options.image_mode.currentIndex()
         state['image_zoom_mode'] = self.options.zoom_mode.currentIndex()
         state['word_wrap'] = self.options.enable_word_wrapping.isChecked()
@@ -538,6 +578,9 @@ class Viewer(QtWidgets.QFrame):
         diff_numbers = bool(state.get('show_diff_line_numbers', False))
         self.set_line_numbers(diff_numbers, update=True)
 
+        show_filenames = bool(state.get('show_diff_filenames', True))
+        self.set_show_filenames(show_filenames, update=True)
+
         image_mode = utils.asint(state.get('image_diff_mode', 0))
         self.options.image_mode.set_index(image_mode)
 
@@ -564,9 +607,29 @@ class Viewer(QtWidgets.QFrame):
         # The "file type" is whether the file itself is an image.
         self.options.set_file_type(file_type)
 
+    def enable_filename_tracking(self):
+        """Enable displaying the currently selected filename"""
+        self.context.selection.selection_changed.connect(
+            self.update_filename, type=Qt.QueuedConnection
+        )
+
+    def update_filename(self):
+        """Update the filename display when the selection changes"""
+        filename = self.context.selection.filename()
+        self.filename.set_text(filename or '')
+
     def update_options(self):
         """Emit a signal indicating that options have changed"""
         self.text.update_options()
+        show_filenames = get(self.options.show_filenames)
+        self.set_show_filenames(show_filenames)
+
+    def set_show_filenames(self, enabled, update=False):
+        """Enable/disable displaying the selected filename"""
+        self.filename.setVisible(enabled)
+        if update:
+            with qtutils.BlockSignals(self.options.show_filenames):
+                self.options.show_filenames.setChecked(enabled)
 
     def set_line_numbers(self, enabled, update=False):
         """Enable/disable line numbers in the text widget"""
@@ -581,7 +644,7 @@ class Viewer(QtWidgets.QFrame):
         self.cleanup()
 
     def cleanup(self):
-        for (image, unlink) in self.images:
+        for image, unlink in self.images:
             if unlink and core.exists(image):
                 os.unlink(image)
         self.images = []
@@ -709,26 +772,23 @@ class Options(QtWidgets.QWidget):
     PIXEL_XOR = 3
 
     def __init__(self, parent):
-        super(Options, self).__init__(parent)
+        super().__init__(parent)
         # Create widgets
         self.widget = parent
         self.ignore_space_at_eol = self.add_option(
             N_('Ignore changes in whitespace at EOL')
         )
-
         self.ignore_space_change = self.add_option(
             N_('Ignore changes in amount of whitespace')
         )
-
         self.ignore_all_space = self.add_option(N_('Ignore all whitespace'))
-
         self.function_context = self.add_option(
             N_('Show whole surrounding functions of changes')
         )
-
         self.show_line_numbers = qtutils.add_action_bool(
             self, N_('Show line numbers'), self.set_line_numbers, True
         )
+        self.show_filenames = self.add_option(N_('Show filenames'))
         self.enable_word_wrapping = qtutils.add_action_bool(
             self, N_('Enable word wrapping'), self.set_word_wrapping, True
         )
@@ -766,6 +826,7 @@ class Options(QtWidgets.QWidget):
         menu.addSeparator()
         menu.addAction(self.function_context)
         menu.addAction(self.show_line_numbers)
+        menu.addAction(self.show_filenames)
         menu.addSeparator()
         menu.addAction(self.enable_word_wrapping)
 
@@ -831,6 +892,7 @@ class Options(QtWidgets.QWidget):
 
     def hide_advanced_options(self):
         """Hide advanced options that are not applicable to the DiffWidget"""
+        self.show_filenames.setVisible(False)
         self.show_line_numbers.setVisible(False)
         self.ignore_space_at_eol.setVisible(False)
         self.ignore_space_change.setVisible(False)
@@ -838,9 +900,7 @@ class Options(QtWidgets.QWidget):
         self.function_context.setVisible(False)
 
 
-# pylint: disable=too-many-ancestors
 class DiffEditor(DiffTextEdit):
-
     up = Signal()
     down = Signal()
     options_changed = Signal()
@@ -863,7 +923,7 @@ class DiffEditor(DiffTextEdit):
         )
 
         self.action_revert_selection = qtutils.add_action(
-            self, 'Revert', self.revert_selection, hotkeys.REVERT
+            self, 'Revert', self.revert_selection, hotkeys.REVERT, hotkeys.REVERT_ALT
         )
         self.action_revert_selection.setIcon(icons.undo())
 
@@ -931,7 +991,7 @@ class DiffEditor(DiffTextEdit):
 
     def create_context_menu(self, event_pos):
         """Override create_context_menu() to display a completely custom menu"""
-        menu = super(DiffEditor, self).create_context_menu(event_pos)
+        menu = super().create_context_menu(event_pos)
         context = self.context
         model = self.model
         s = self.selection_model.selection()
@@ -941,15 +1001,22 @@ class DiffEditor(DiffTextEdit):
         current_actions = menu.actions()
         menu_actions = []
         add_action = menu_actions.append
+        edit_actions_added = False
+        stage_action_added = False
 
-        if model.is_stageable() or model.is_unstageable():
-            if model.is_stageable():
-                self.stage_or_unstage.setText(N_('Stage'))
-                self.stage_or_unstage.setIcon(icons.add())
-            else:
-                self.stage_or_unstage.setText(N_('Unstage'))
-                self.stage_or_unstage.setIcon(icons.remove())
-            add_action(self.stage_or_unstage)
+        if s.staged and model.is_unstageable():
+            item = s.staged[0]
+            if item not in model.submodules and item not in model.staged_deleted:
+                if self.has_selection():
+                    apply_text = N_('Unstage Selected Lines')
+                else:
+                    apply_text = N_('Unstage Diff Hunk')
+                self.action_apply_selection.setText(apply_text)
+                self.action_apply_selection.setIcon(icons.remove())
+                add_action(self.action_apply_selection)
+                stage_action_added = self._add_stage_or_unstage_action(
+                    menu, add_action, stage_action_added
+                )
 
         if model.is_partially_stageable():
             item = s.modified[0] if s.modified else None
@@ -963,6 +1030,9 @@ class DiffEditor(DiffTextEdit):
                     hotkeys.STAGE_SELECTION,
                 )
                 add_action(action)
+                stage_action_added = self._add_stage_or_unstage_action(
+                    menu, add_action, stage_action_added
+                )
 
                 action = qtutils.add_action_with_icon(
                     menu,
@@ -971,7 +1041,7 @@ class DiffEditor(DiffTextEdit):
                     cmds.run(cmds.OpenRepo, context, path),
                 )
                 add_action(action)
-            elif item not in model.unstaged_deleted:
+            elif item and item not in model.unstaged_deleted:
                 if self.has_selection():
                     apply_text = N_('Stage Selected Lines')
                     edit_and_apply_text = N_('Edit Selected Lines to Stage...')
@@ -987,13 +1057,25 @@ class DiffEditor(DiffTextEdit):
                 self.action_apply_selection.setIcon(icons.add())
                 add_action(self.action_apply_selection)
 
+                self.action_revert_selection.setText(revert_text)
+                add_action(self.action_revert_selection)
+
+                stage_action_added = self._add_stage_or_unstage_action(
+                    menu, add_action, stage_action_added
+                )
+                # Do not show the "edit" action when the file does not exist.
+                add_action(qtutils.menu_separator(menu))
+                if filename and core.exists(filename):
+                    add_action(self.launch_editor)
+                # Removed files can still be diffed.
+                add_action(self.launch_difftool)
+                edit_actions_added = True
+
+                add_action(qtutils.menu_separator(menu))
                 self.action_edit_and_apply_selection.setText(edit_and_apply_text)
                 self.action_edit_and_apply_selection.setIcon(icons.add())
                 add_action(self.action_edit_and_apply_selection)
 
-                self.action_revert_selection.setText(revert_text)
-                add_action(self.action_revert_selection)
-
                 self.action_edit_and_revert_selection.setText(edit_and_revert_text)
                 add_action(self.action_edit_and_revert_selection)
 
@@ -1010,6 +1092,10 @@ class DiffEditor(DiffTextEdit):
                 )
                 add_action(action)
 
+                stage_action_added = self._add_stage_or_unstage_action(
+                    menu, add_action, stage_action_added
+                )
+
                 qtutils.add_action_with_icon(
                     menu,
                     icons.cola(),
@@ -1019,31 +1105,36 @@ class DiffEditor(DiffTextEdit):
                 add_action(action)
 
             elif item not in model.staged_deleted:
+                # Do not show the "edit" action when the file does not exist.
+                add_action(qtutils.menu_separator(menu))
+                if filename and core.exists(filename):
+                    add_action(self.launch_editor)
+                # Removed files can still be diffed.
+                add_action(self.launch_difftool)
+                add_action(qtutils.menu_separator(menu))
+                edit_actions_added = True
+
                 if self.has_selection():
-                    apply_text = N_('Unstage Selected Lines')
                     edit_and_apply_text = N_('Edit Selected Lines to Unstage...')
                 else:
-                    apply_text = N_('Unstage Diff Hunk')
                     edit_and_apply_text = N_('Edit Diff Hunk to Unstage...')
-
-                self.action_apply_selection.setText(apply_text)
-                self.action_apply_selection.setIcon(icons.remove())
-                add_action(self.action_apply_selection)
-
                 self.action_edit_and_apply_selection.setText(edit_and_apply_text)
                 self.action_edit_and_apply_selection.setIcon(icons.remove())
                 add_action(self.action_edit_and_apply_selection)
 
-        if model.is_stageable() or model.is_unstageable():
+        if not edit_actions_added and (model.is_stageable() or model.is_unstageable()):
+            add_action(qtutils.menu_separator(menu))
             # Do not show the "edit" action when the file does not exist.
             # Untracked files exist by definition.
             if filename and core.exists(filename):
-                add_action(qtutils.menu_separator(menu))
                 add_action(self.launch_editor)
 
             # Removed files can still be diffed.
             add_action(self.launch_difftool)
 
+        add_action(qtutils.menu_separator(menu))
+        _add_patch_actions(self, self.context, menu)
+
         # Add the Previous/Next File actions, which improves discoverability
         # of their associated shortcuts
         add_action(qtutils.menu_separator(menu))
@@ -1059,6 +1150,23 @@ class DiffEditor(DiffTextEdit):
 
         return menu
 
+    def _add_stage_or_unstage_action(self, menu, add_action, already_added):
+        """Add the Stage / Unstage menu action"""
+        if already_added:
+            return True
+        model = self.context.model
+        s = self.selection_model.selection()
+        if model.is_stageable() or model.is_unstageable():
+            if (model.is_amend_mode() and s.staged) or not self.model.is_stageable():
+                self.stage_or_unstage.setText(N_('Unstage'))
+                self.stage_or_unstage.setIcon(icons.remove())
+            else:
+                self.stage_or_unstage.setText(N_('Stage'))
+                self.stage_or_unstage.setIcon(icons.add())
+            add_action(qtutils.menu_separator(menu))
+            add_action(self.stage_or_unstage)
+        return True
+
     def mousePressEvent(self, event):
         if event.button() == Qt.RightButton:
             # Intercept right-click to move the cursor to the current position.
@@ -1068,7 +1176,7 @@ class DiffEditor(DiffTextEdit):
                 cursor = self.cursorForPosition(event.pos())
                 self.setTextCursor(cursor)
 
-        return super(DiffEditor, self).mousePressEvent(event)
+        return super().mousePressEvent(event)
 
     def setPlainText(self, text):
         """setPlainText(str) while retaining scrollbar positions"""
@@ -1176,6 +1284,122 @@ class DiffEditor(DiffTextEdit):
         self.selection_model.line_number = self.numbers.current_line()
 
 
+def _add_patch_actions(widget, context, menu):
+    """Add actions for manipulating patch files"""
+    patches_menu = menu.addMenu(N_('Patches'))
+    patches_menu.setIcon(icons.diff())
+    export_action = qtutils.add_action(
+        patches_menu,
+        N_('Export Patch'),
+        lambda: _export_patch(widget, context),
+    )
+    export_action.setIcon(icons.save())
+    patches_menu.addAction(export_action)
+
+    # Build the "Append Patch" menu dynamically.
+    append_menu = patches_menu.addMenu(N_('Append Patch'))
+    append_menu.setIcon(icons.add())
+    append_menu.aboutToShow.connect(
+        lambda: _build_patch_append_menu(widget, context, append_menu)
+    )
+
+
+def _build_patch_append_menu(widget, context, menu):
+    """Build the "Append Patch" sub-menu"""
+    # Build the menu when first displayed only. This initial check avoids
+    # re-populating the menu with duplicate actions.
+    menu_actions = menu.actions()
+    if menu_actions:
+        return
+
+    choose_patch_action = qtutils.add_action(
+        menu,
+        N_('Choose Patch...'),
+        lambda: _export_patch(widget, context, append=True),
+    )
+    choose_patch_action.setIcon(icons.diff())
+    menu.addAction(choose_patch_action)
+
+    subdir_menus = {}
+    path = prefs.patches_directory(context)
+    patches = get_patches_from_dir(path)
+    for patch in patches:
+        relpath = os.path.relpath(patch, start=path)
+        sub_menu = _add_patch_subdirs(menu, subdir_menus, relpath)
+        patch_basename = os.path.basename(relpath)
+        append_action = qtutils.add_action(
+            sub_menu,
+            patch_basename,
+            lambda patch_file=patch: _append_patch(widget, patch_file),
+        )
+        append_action.setIcon(icons.save())
+        sub_menu.addAction(append_action)
+
+
+def _add_patch_subdirs(menu, subdir_menus, relpath):
+    """Build menu leading up to the patch"""
+    # If the path contains no directory separators then add it to the
+    # root of the menu.
+    if os.sep not in relpath:
+        return menu
+
+    # Loop over each directory component and build a menu if it doesn't already exist.
+    components = []
+    for dirname in os.path.dirname(relpath).split(os.sep):
+        components.append(dirname)
+        current_dir = os.sep.join(components)
+        try:
+            menu = subdir_menus[current_dir]
+        except KeyError:
+            menu = subdir_menus[current_dir] = menu.addMenu(dirname)
+            menu.setIcon(icons.folder())
+
+    return menu
+
+
+def _export_patch(diff_editor, context, append=False):
+    """Export the selected diff to a patch file"""
+    if diff_editor.selection_model.is_empty():
+        return
+    patch = diff_editor.extract_patch(reverse=False)
+    if not patch.has_changes():
+        return
+    directory = prefs.patches_directory(context)
+    if append:
+        filename = qtutils.existing_file(directory, title=N_('Append Patch...'))
+    else:
+        default_filename = os.path.join(directory, 'diff.patch')
+        filename = qtutils.save_as(default_filename)
+    if not filename:
+        return
+    _write_patch_to_file(diff_editor, patch, filename, append=append)
+
+
+def _append_patch(diff_editor, filename):
+    """Append diffs to the specified patch file"""
+    if diff_editor.selection_model.is_empty():
+        return
+    patch = diff_editor.extract_patch(reverse=False)
+    if not patch.has_changes():
+        return
+    _write_patch_to_file(diff_editor, patch, filename, append=True)
+
+
+def _write_patch_to_file(diff_editor, patch, filename, append=False):
+    """Write diffs from the Diff Editor to the specified patch file"""
+    encoding = diff_editor.patch_encoding()
+    content = patch.as_text()
+    try:
+        core.write(filename, content, encoding=encoding, append=append)
+    except OSError as exc:
+        _, details = utils.format_exception(exc)
+        title = N_('Error writing patch')
+        msg = N_('Unable to write patch to "%s". Check permissions?' % filename)
+        Interaction.critical(title, message=msg, details=details)
+        return
+    Interaction.log('Patch written to "%s"' % filename)
+
+
 class DiffWidget(QtWidgets.QWidget):
     """Display commit metadata and text diffs"""
 
@@ -1194,48 +1418,36 @@ class DiffWidget(QtWidgets.QWidget):
         summary_font = QtGui.QFont(author_font)
         summary_font.setWeight(QtGui.QFont.Bold)
 
-        policy = QtWidgets.QSizePolicy(
-            QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Minimum
-        )
-
         self.gravatar_label = gravatar.GravatarLabel(self.context, parent=self)
 
-        self.author_label = TextLabel()
-        self.author_label.setTextFormat(Qt.RichText)
+        self.oid_label = PlainTextLabel(parent=self)
+        self.oid_label.setAlignment(Qt.AlignBottom)
+        self.oid_label.elide()
+
+        self.author_label = RichTextLabel(parent=self)
         self.author_label.setFont(author_font)
-        self.author_label.setSizePolicy(policy)
-        self.author_label.setAlignment(Qt.AlignBottom)
+        self.author_label.setAlignment(Qt.AlignTop)
         self.author_label.elide()
 
-        self.date_label = TextLabel()
-        self.date_label.setTextFormat(Qt.PlainText)
-        self.date_label.setSizePolicy(policy)
+        self.date_label = PlainTextLabel(parent=self)
         self.date_label.setAlignment(Qt.AlignTop)
-        self.date_label.hide()
+        self.date_label.elide()
 
-        self.summary_label = TextLabel()
-        self.summary_label.setTextFormat(Qt.PlainText)
+        self.summary_label = PlainTextLabel(parent=self)
         self.summary_label.setFont(summary_font)
-        self.summary_label.setSizePolicy(policy)
         self.summary_label.setAlignment(Qt.AlignTop)
         self.summary_label.elide()
 
-        self.oid_label = TextLabel()
-        self.oid_label.setTextFormat(Qt.PlainText)
-        self.oid_label.setSizePolicy(policy)
-        self.oid_label.setAlignment(Qt.AlignTop)
-        self.oid_label.elide()
-
         self.diff = DiffTextEdit(context, self, is_commit=is_commit, whitespace=False)
         self.setFocusProxy(self.diff)
 
         self.info_layout = qtutils.vbox(
             defs.no_margin,
             defs.no_spacing,
+            self.oid_label,
             self.author_label,
             self.date_label,
             self.summary_label,
-            self.oid_label,
         )
 
         self.logo_layout = qtutils.hbox(
@@ -1284,10 +1496,11 @@ class DiffWidget(QtWidgets.QWidget):
             return
         commit = commits[-1]
         oid = commit.oid
-        email = commit.email or ''
-        summary = commit.summary or ''
         author = commit.author or ''
-        self.set_details(oid, author, email, '', summary)
+        email = commit.email or ''
+        date = commit.authdate or ''
+        summary = commit.summary or ''
+        self.set_details(oid, author, email, date, summary)
         self.oid = oid
 
         if len(commits) > 1:
@@ -1330,21 +1543,22 @@ class DiffWidget(QtWidgets.QWidget):
 
     def files_selected(self, filenames):
         """Update the view when a filename is selected"""
-        if not filenames:
-            return
         oid_start = self.oid_start
         oid_end = self.oid_end
+        extra_args = {}
+        if filenames:
+            extra_args['filename'] = filenames[0]
         if oid_start and oid_end:
-            self.set_diff_range(oid_start.oid, oid_end.oid, filename=filenames[0])
+            self.set_diff_range(oid_start.oid, oid_end.oid, **extra_args)
         else:
-            self.set_diff_oid(self.oid, filename=filenames[0])
+            self.set_diff_oid(self.oid, **extra_args)
 
 
 class DiffPanel(QtWidgets.QWidget):
     """A combined diff + search panel"""
 
     def __init__(self, diff_widget, text_widget, parent):
-        super(DiffPanel, self).__init__(parent)
+        super().__init__(parent)
         self.diff_widget = diff_widget
         self.search_widget = TextSearchWidget(text_widget, self)
         self.search_widget.hide()
@@ -1366,54 +1580,7 @@ class DiffPanel(QtWidgets.QWidget):
         # The diff search is only active in text mode.
         if not self.search_widget.isVisible():
             self.search_widget.show()
-        self.search_widget.setFocus(True)
-
-
-class TextLabel(QtWidgets.QLabel):
-    def __init__(self, parent=None):
-        QtWidgets.QLabel.__init__(self, parent)
-        self.setTextInteractionFlags(
-            Qt.TextSelectableByMouse | Qt.LinksAccessibleByMouse
-        )
-        self._display = ''
-        self._template = ''
-        self._text = ''
-        self._elide = False
-        self._metrics = QtGui.QFontMetrics(self.font())
-        self.setOpenExternalLinks(True)
-
-    def elide(self):
-        self._elide = True
-
-    def set_text(self, text):
-        self.set_template(text, text)
-
-    def set_template(self, text, template):
-        self._display = text
-        self._text = text
-        self._template = template
-        self.update_text(self.width())
-        self.setText(self._display)
-
-    def update_text(self, width):
-        self._display = self._text
-        if not self._elide:
-            return
-        text = self._metrics.elidedText(self._template, Qt.ElideRight, width - 2)
-        if text != self._template:
-            self._display = text
-
-    # Qt overrides
-    def setFont(self, font):
-        self._metrics = QtGui.QFontMetrics(font)
-        QtWidgets.QLabel.setFont(self, font)
-
-    def resizeEvent(self, event):
-        if self._elide:
-            self.update_text(event.size().width())
-            with qtutils.BlockSignals(self):
-                self.setText(self._display)
-        QtWidgets.QLabel.resizeEvent(self, event)
+        self.search_widget.setFocus()
 
 
 class DiffInfoTask(qtutils.Task):
@@ -1444,3 +1611,288 @@ class DiffRangeTask(qtutils.Task):
     def task(self):
         context = self.context
         return gitcmds.diff_range(context, self.start, self.end, filename=self.filename)
+
+
+def apply_patches(context, patches=None):
+    """Open the ApplyPatches dialog"""
+    parent = qtutils.active_window()
+    dlg = new_apply_patches(context, patches=patches, parent=parent)
+    dlg.show()
+    dlg.raise_()
+    return dlg
+
+
+def new_apply_patches(context, patches=None, parent=None):
+    """Create a new instances of the ApplyPatches dialog"""
+    dlg = ApplyPatches(context, parent=parent)
+    if patches:
+        dlg.add_paths(patches)
+    return dlg
+
+
+def get_patches_from_paths(paths):
+    """Returns all patches beneath a given path"""
+    paths = [core.decode(p) for p in paths]
+    patches = [p for p in paths if core.isfile(p) and p.endswith(('.patch', '.mbox'))]
+    dirs = [p for p in paths if core.isdir(p)]
+    dirs.sort()
+    for d in dirs:
+        patches.extend(get_patches_from_dir(d))
+    return patches
+
+
+def get_patches_from_mimedata(mimedata):
+    """Extract path files from a QMimeData payload"""
+    urls = mimedata.urls()
+    if not urls:
+        return []
+    paths = [x.path() for x in urls]
+    return get_patches_from_paths(paths)
+
+
+def get_patches_from_dir(path):
+    """Find patches in a subdirectory"""
+    patches = []
+    for root, _, files in core.walk(path):
+        for name in [f for f in files if f.endswith(('.patch', '.mbox'))]:
+            patches.append(core.decode(os.path.join(root, name)))
+    return patches
+
+
+class ApplyPatches(standard.Dialog):
+    def __init__(self, context, parent=None):
+        super().__init__(parent=parent)
+        self.context = context
+        self.setWindowTitle(N_('Apply Patches'))
+        self.setAcceptDrops(True)
+        if parent is not None:
+            self.setWindowModality(Qt.WindowModal)
+
+        self.curdir = core.getcwd()
+        self.inner_drag = False
+
+        self.usage = QtWidgets.QLabel()
+        self.usage.setText(
+            N_(
+                """
+            <p>
+                Drag and drop or use the <strong>Add</strong> button to add
+                patches to the list
+            </p>
+            """
+            )
+        )
+
+        self.tree = PatchTreeWidget(parent=self)
+        self.tree.setHeaderHidden(True)
+        self.tree.itemSelectionChanged.connect(self._tree_selection_changed)
+
+        self.diffwidget = DiffWidget(context, self, is_commit=True)
+
+        self.add_button = qtutils.create_toolbutton(
+            text=N_('Add'), icon=icons.add(), tooltip=N_('Add patches (+)')
+        )
+
+        self.remove_button = qtutils.create_toolbutton(
+            text=N_('Remove'),
+            icon=icons.remove(),
+            tooltip=N_('Remove selected (Delete)'),
+        )
+
+        self.apply_button = qtutils.create_button(text=N_('Apply'), icon=icons.ok())
+
+        self.close_button = qtutils.close_button()
+
+        self.add_action = qtutils.add_action(
+            self, N_('Add'), self.add_files, hotkeys.ADD_ITEM
+        )
+
+        self.remove_action = qtutils.add_action(
+            self,
+            N_('Remove'),
+            self.tree.remove_selected,
+            hotkeys.DELETE,
+            hotkeys.BACKSPACE,
+            hotkeys.REMOVE_ITEM,
+        )
+
+        self.top_layout = qtutils.hbox(
+            defs.no_margin,
+            defs.button_spacing,
+            self.add_button,
+            self.remove_button,
+            qtutils.STRETCH,
+            self.usage,
+        )
+
+        self.bottom_layout = qtutils.hbox(
+            defs.no_margin,
+            defs.button_spacing,
+            qtutils.STRETCH,
+            self.close_button,
+            self.apply_button,
+        )
+
+        self.splitter = qtutils.splitter(Qt.Vertical, self.tree, self.diffwidget)
+
+        self.main_layout = qtutils.vbox(
+            defs.margin,
+            defs.spacing,
+            self.top_layout,
+            self.splitter,
+            self.bottom_layout,
+        )
+        self.setLayout(self.main_layout)
+
+        qtutils.connect_button(self.add_button, self.add_files)
+        qtutils.connect_button(self.remove_button, self.tree.remove_selected)
+        qtutils.connect_button(self.apply_button, self.apply_patches)
+        qtutils.connect_button(self.close_button, self.close)
+
+        self.init_state(None, self.resize, 720, 480)
+
+    def apply_patches(self):
+        items = self.tree.items()
+        if not items:
+            return
+        context = self.context
+        patches = [i.data(0, Qt.UserRole) for i in items]
+        cmds.do(cmds.ApplyPatches, context, patches)
+        self.accept()
+
+    def add_files(self):
+        files = qtutils.open_files(
+            N_('Select patch file(s)...'),
+            directory=self.curdir,
+            filters='Patches (*.patch *.mbox)',
+        )
+        if not files:
+            return
+        self.curdir = os.path.dirname(files[0])
+        self.add_paths([core.relpath(f) for f in files])
+
+    def dragEnterEvent(self, event):
+        """Accepts drops if the mimedata contains patches"""
+        super().dragEnterEvent(event)
+        patches = get_patches_from_mimedata(event.mimeData())
+        if patches:
+            event.acceptProposedAction()
+
+    def dropEvent(self, event):
+        """Add dropped patches"""
+        event.accept()
+        patches = get_patches_from_mimedata(event.mimeData())
+        if not patches:
+            return
+        self.add_paths(patches)
+
+    def add_paths(self, paths):
+        self.tree.add_paths(paths)
+
+    def _tree_selection_changed(self):
+        items = self.tree.selected_items()
+        if not items:
+            return
+        item = items[-1]  # take the last item
+        path = item.data(0, Qt.UserRole)
+        if not core.exists(path):
+            return
+        commit = parse_patch(path)
+        self.diffwidget.set_details(
+            commit.oid, commit.author, commit.email, commit.date, commit.summary
+        )
+        self.diffwidget.set_diff(commit.diff)
+
+    def export_state(self):
+        """Export persistent settings"""
+        state = super().export_state()
+        state['sizes'] = get(self.splitter)
+        return state
+
+    def apply_state(self, state):
+        """Apply persistent settings"""
+        result = super().apply_state(state)
+        try:
+            self.splitter.setSizes(state['sizes'])
+        except (AttributeError, KeyError, ValueError, TypeError):
+            pass
+        return result
+
+
+class PatchTreeWidget(standard.DraggableTreeWidget):
+    def add_paths(self, paths):
+        patches = get_patches_from_paths(paths)
+        if not patches:
+            return
+        items = []
+        icon = icons.file_text()
+        for patch in patches:
+            item = QtWidgets.QTreeWidgetItem()
+            flags = item.flags() & ~Qt.ItemIsDropEnabled
+            item.setFlags(flags)
+            item.setIcon(0, icon)
+            item.setText(0, os.path.basename(patch))
+            item.setData(0, Qt.UserRole, patch)
+            item.setToolTip(0, patch)
+            items.append(item)
+        self.addTopLevelItems(items)
+
+    def remove_selected(self):
+        idxs = self.selectedIndexes()
+        rows = [idx.row() for idx in idxs]
+        for row in reversed(sorted(rows)):
+            self.invisibleRootItem().takeChild(row)
+
+
+class Commit:
+    """Container for commit details"""
+
+    def __init__(self):
+        self.content = ''
+        self.author = ''
+        self.email = ''
+        self.oid = ''
+        self.summary = ''
+        self.diff = ''
+        self.date = ''
+
+
+def parse_patch(path):
+    content = core.read(path)
+    commit = Commit()
+    parse(content, commit)
+    return commit
+
+
+def parse(content, commit):
+    """Parse commit details from a patch"""
+    from_rgx = re.compile(r'^From (?P<oid>[a-f0-9]{40}) .*$')
+    author_rgx = re.compile(r'^From: (?P<author>[^<]+) <(?P<email>[^>]+)>$')
+    date_rgx = re.compile(r'^Date: (?P<date>.*)$')
+    subject_rgx = re.compile(r'^Subject: (?P<summary>.*)$')
+
+    commit.content = content
+
+    lines = content.splitlines()
+    for idx, line in enumerate(lines):
+        match = from_rgx.match(line)
+        if match:
+            commit.oid = match.group('oid')
+            continue
+
+        match = author_rgx.match(line)
+        if match:
+            commit.author = match.group('author')
+            commit.email = match.group('email')
+            continue
+
+        match = date_rgx.match(line)
+        if match:
+            commit.date = match.group('date')
+            continue
+
+        match = subject_rgx.match(line)
+        if match:
+            commit.summary = match.group('summary')
+            commit.diff = '\n'.join(lines[idx + 1 :])
+            break
