@@ -1,8 +1,7 @@
-from __future__ import absolute_import, division, print_function, unicode_literals
-
 from qtpy import QtWidgets
 from qtpy.QtCore import Signal
 from qtpy.QtCore import QSize
+from qtpy.QtCore import Qt
 
 from .. import cmds
 from .. import hotkeys
@@ -11,15 +10,14 @@ from ..i18n import N_
 from .standard import TreeWidget
 
 
-# pylint: disable=too-many-ancestors
 class FileWidget(TreeWidget):
-
     files_selected = Signal(object)
     difftool_selected = Signal(object)
     histories_selected = Signal(object)
     grab_file = Signal(object)
+    remark_toggled = Signal(object, object)
 
-    def __init__(self, context, parent):
+    def __init__(self, context, parent, remarks=False):
         TreeWidget.__init__(self, parent)
         self.context = context
 
@@ -42,7 +40,19 @@ class FileWidget(TreeWidget):
             self, N_('Grab File...'), self._grab_file
         )
 
-        # pylint: disable=no-member
+        if remarks:
+            self.toggle_remark_actions = tuple(
+                qtutils.add_action(
+                    self,
+                    r,
+                    lambda remark=r: self.toggle_remark(remark),
+                    hotkeys.hotkey(Qt.CTRL | getattr(Qt, 'Key_' + r)),
+                )
+                for r in map(str, range(10))
+            )
+        else:
+            self.toggle_remark_actions = tuple()
+
         self.itemSelectionChanged.connect(self.selection_changed)
 
     def selection_changed(self):
@@ -112,6 +122,9 @@ class FileWidget(TreeWidget):
         menu.addAction(self.show_history_action)
         menu.addAction(self.launch_difftool_action)
         menu.addAction(self.launch_editor_action)
+        if self.toggle_remark_actions:
+            menu_toggle_remark = menu.addMenu(N_('Toggle remark of touching commits'))
+            tuple(map(menu_toggle_remark.addAction, self.toggle_remark_actions))
         menu.exec_(self.mapToGlobal(event.pos()))
 
     def show_diff(self):
@@ -132,6 +145,11 @@ class FileWidget(TreeWidget):
         paths = [i.path for i in items]
         self.histories_selected.emit(paths)
 
+    def toggle_remark(self, remark):
+        items = self.selected_items()
+        paths = tuple(i.path for i in items)
+        self.remark_toggled.emit(remark, paths)
+
 
 class FileTreeWidgetItem(QtWidgets.QTreeWidgetItem):
     def __init__(self, file_log, parent=None):
