@@ -1,5 +1,4 @@
 """Widgets for Fetch, Push, and Pull"""
-from __future__ import absolute_import, division, print_function, unicode_literals
 import fnmatch
 
 from qtpy import QtGui
@@ -15,6 +14,7 @@ from .. import icons
 from .. import qtutils
 from .. import utils
 from . import defs
+from . import log
 from . import standard
 
 
@@ -132,8 +132,6 @@ class RemoteActionDialog(standard.Dialog):
         self.selected_remotes = []
 
         self.runtask = qtutils.RunTask(parent=self)
-        self.progress = standard.progress(title, N_('Updating'), self)
-
         self.local_label = QtWidgets.QLabel()
         self.local_label.setText(N_('Local Branch'))
 
@@ -148,7 +146,7 @@ class RemoteActionDialog(standard.Dialog):
 
         self.remote_name = QtWidgets.QLineEdit()
         qtutils.add_completer(self.remote_name, model.remotes)
-        # pylint: disable=no-member
+
         self.remote_name.editingFinished.connect(self.remote_name_edited)
         self.remote_name.textEdited.connect(lambda x: self.remote_name_edited())
 
@@ -174,6 +172,12 @@ class RemoteActionDialog(standard.Dialog):
             checked=True, text=text, tooltip=tooltip
         )
 
+        text = N_('Show remote messages')
+        tooltip = N_('Display remote messages in a separate dialog')
+        self.remote_messages_checkbox = qtutils.checkbox(
+            checked=False, text=text, tooltip=tooltip
+        )
+
         text = N_('Fast-forward only')
         tooltip = N_(
             'Refuse to merge unless the current HEAD is already up-'
@@ -214,10 +218,36 @@ class RemoteActionDialog(standard.Dialog):
         tooltip = N_('Configure the remote branch as the the new upstream')
         self.upstream_checkbox = qtutils.checkbox(text=text, tooltip=tooltip)
 
+        text = N_('Close on completion')
+        tooltip = N_('Close dialog when completed')
+        self.close_on_completion_checkbox = qtutils.checkbox(
+            checked=True, text=text, tooltip=tooltip
+        )
+
         self.action_button = qtutils.ok_button(title, icon=icon)
         self.close_button = qtutils.close_button()
-
-        self.buttons = utils.Group(self.action_button, self.close_button)
+        self.buttons_group = utils.Group(self.close_button, self.action_button)
+        self.inputs_group = utils.Group(
+            self.close_on_completion_checkbox,
+            self.force_checkbox,
+            self.ff_only_checkbox,
+            self.local_branch,
+            self.local_branches,
+            self.tags_checkbox,
+            self.prune_checkbox,
+            self.rebase_checkbox,
+            self.remote_name,
+            self.remotes,
+            self.remote_branch,
+            self.remote_branches,
+            self.upstream_checkbox,
+            self.prompt_checkbox,
+            self.remote_messages_checkbox,
+        )
+        self.progress = standard.progress_bar(
+            self,
+            disable=(self.buttons_group, self.inputs_group),
+        )
 
         self.local_branch_layout = qtutils.hbox(
             defs.small_margin, defs.spacing, self.local_label, self.local_branch
@@ -245,7 +275,10 @@ class RemoteActionDialog(standard.Dialog):
             self.rebase_checkbox,
             self.upstream_checkbox,
             self.prompt_checkbox,
+            self.close_on_completion_checkbox,
+            self.remote_messages_checkbox,
             qtutils.STRETCH,
+            self.progress,
             self.close_button,
             self.action_button,
         )
@@ -300,7 +333,6 @@ class RemoteActionDialog(standard.Dialog):
         self.set_field_defaults()
 
         # Setup signals and slots
-        # pylint: disable=no-member
         self.remotes.itemSelectionChanged.connect(self.update_remotes)
 
         local = self.local_branches
@@ -460,7 +492,7 @@ class RemoteActionDialog(standard.Dialog):
         displayed = []
         for remote_name in self.model.remotes:
             url = self.model.remote_url(remote_name, self.action)
-            display = '%s\t(%s)' % (remote_name, N_('URL: %s') % url)
+            display = '{}\t({})'.format(remote_name, N_('URL: %s') % url)
             displayed.append(display)
         qtutils.set_items(widget, displayed)
 
@@ -565,6 +597,7 @@ class RemoteActionDialog(standard.Dialog):
     def action_callback(self):
         """Perform the actual fetch/push/pull operation"""
         action = self.action
+        remote_messages = get(self.remote_messages_checkbox)
         if action == FETCH:
             model_action = self.model.fetch
         elif action == PUSH:
@@ -586,12 +619,15 @@ class RemoteActionDialog(standard.Dialog):
 
         if action == PUSH and not remote_branch:
             branch = local_branch
-            candidate = '%s/%s' % (remote, branch)
+            candidate = f'{remote}/{branch}'
             prompt = get(self.prompt_checkbox)
 
             if prompt and candidate not in self.model.remote_branches:
                 title = N_('Push')
-                args = dict(branch=branch, remote=remote)
+                args = {
+                    'branch': branch,
+                    'remote': remote,
+                }
                 msg = (
                     N_(
                         'Branch "%(branch)s" does not exist in "%(remote)s".\n'
@@ -627,18 +663,29 @@ class RemoteActionDialog(standard.Dialog):
             ):
                 return
 
-        # Disable the GUI by default
-        self.buttons.setEnabled(False)
+        self.progress.setMaximumHeight(
+            self.action_button.height() - defs.small_margin * 2
+        )
 
         # Use a thread to update in the background
         task = ActionTask(model_action, remote, kwargs)
-        self.runtask.start(task, progress=self.progress, finish=self.action_completed)
+        if remote_messages:
+            result = log.show_remote_messages(self, self.context)
+        else:
+            result = None
+        self.runtask.start(
+            task,
+            progress=self.progress,
+            finish=self.action_completed,
+            result=result,
+        )
 
     def action_completed(self, task):
         """Grab the results of the action and finish up"""
-        status, out, err = task.result
-        self.buttons.setEnabled(True)
+        if not task.result or not isinstance(task.result, (list, tuple)):
+            return
 
+        status, out, err = task.result
         command = 'git %s' % self.action.lower()
         message = Interaction.format_command_status(command, status)
         details = Interaction.format_out_err(out, err)
@@ -649,7 +696,9 @@ class RemoteActionDialog(standard.Dialog):
         Interaction.log(log_message)
 
         if status == 0:
-            self.accept()
+            close_on_completion = get(self.close_on_completion_checkbox)
+            if close_on_completion:
+                self.accept()
             return
 
         if self.action == PUSH:
@@ -658,15 +707,31 @@ class RemoteActionDialog(standard.Dialog):
 
         Interaction.critical(self.windowTitle(), message=message, details=details)
 
+    def export_state(self):
+        """Export persistent settings"""
+        state = standard.Dialog.export_state(self)
+        state['close_on_completion'] = get(self.close_on_completion_checkbox)
+        state['remote_messages'] = get(self.remote_messages_checkbox)
+        return state
+
+    def apply_state(self, state):
+        """Apply persistent settings"""
+        result = standard.Dialog.apply_state(self, state)
+        # Restore the "close on completion" checkbox
+        close_on_completion = bool(state.get('close_on_completion', True))
+        self.close_on_completion_checkbox.setChecked(close_on_completion)
+        # Restore the "show remote messages" checkbox
+        remote_messages = bool(state.get('remote_messages', False))
+        self.remote_messages_checkbox.setChecked(remote_messages)
+        return result
+
 
 # Use distinct classes so that each saves its own set of preferences
 class Fetch(RemoteActionDialog):
     """Fetch from remote repositories"""
 
     def __init__(self, context, parent=None):
-        super(Fetch, self).__init__(
-            context, FETCH, N_('Fetch'), parent=parent, icon=icons.repo()
-        )
+        super().__init__(context, FETCH, N_('Fetch'), parent=parent, icon=icons.repo())
 
     def export_state(self):
         """Export persistent settings"""
@@ -689,9 +754,7 @@ class Push(RemoteActionDialog):
     """Push to remote repositories"""
 
     def __init__(self, context, parent=None):
-        super(Push, self).__init__(
-            context, PUSH, N_('Push'), parent=parent, icon=icons.push()
-        )
+        super().__init__(context, PUSH, N_('Push'), parent=parent, icon=icons.push())
 
     def export_state(self):
         """Export persistent settings"""
@@ -703,15 +766,12 @@ class Push(RemoteActionDialog):
     def apply_state(self, state):
         """Apply persistent settings"""
         result = RemoteActionDialog.apply_state(self, state)
-
         # Restore the "prompt on creation" checkbox
         prompt = bool(state.get('prompt', True))
         self.prompt_checkbox.setChecked(prompt)
-
         # Restore the "tags" checkbox
         tags = bool(state.get('tags', False))
         self.tags_checkbox.setChecked(tags)
-
         return result
 
 
@@ -719,35 +779,29 @@ class Pull(RemoteActionDialog):
     """Pull from remote repositories"""
 
     def __init__(self, context, parent=None):
-        super(Pull, self).__init__(
-            context, PULL, N_('Pull'), parent=parent, icon=icons.pull()
-        )
+        super().__init__(context, PULL, N_('Pull'), parent=parent, icon=icons.pull())
 
     def apply_state(self, state):
         """Apply persistent settings"""
         result = RemoteActionDialog.apply_state(self, state)
         # Rebase has the highest priority
         rebase = bool(state.get('rebase', False))
+        self.rebase_checkbox.setChecked(rebase)
+
         ff_only = not rebase and bool(state.get('ff_only', False))
         no_ff = not rebase and not ff_only and bool(state.get('no_ff', False))
-
-        self.rebase_checkbox.setChecked(rebase)
         self.no_ff_checkbox.setChecked(no_ff)
-
         # Allow users coming from older versions that have rebase=False to
         # pickup the new ff_only=True default by only setting ff_only False
         # when it either exists in the config or when rebase=True.
         if 'ff_only' in state or rebase:
             self.ff_only_checkbox.setChecked(ff_only)
-
         return result
 
     def export_state(self):
         """Export persistent settings"""
         state = RemoteActionDialog.export_state(self)
-
         state['ff_only'] = get(self.ff_only_checkbox)
         state['no_ff'] = get(self.no_ff_checkbox)
         state['rebase'] = get(self.rebase_checkbox)
-
         return state
