@@ -1,5 +1,4 @@
 """The startup dialog is presented when no repositories can be found at startup"""
-from __future__ import absolute_import, division, print_function, unicode_literals
 
 from qtpy.QtCore import Qt
 from qtpy import QtCore
@@ -132,8 +131,6 @@ class StartupDialog(standard.Dialog):
         qtutils.connect_button(self.clone_button, self.clone_repo)
         qtutils.connect_button(self.new_button, self.new_repo)
         qtutils.connect_button(self.close_button, self.reject)
-
-        # pylint: disable=no-member
         self.tab_bar.currentChanged.connect(self.tab_changed)
 
         self.init_state(settings, self.resize_widget)
@@ -184,12 +181,12 @@ class StartupDialog(standard.Dialog):
             self.context.cfg.set_user('cola.startupmode', list_mode)
 
     def resize_widget(self):
-        screen = QtWidgets.QApplication.instance().desktop()
+        width, height = qtutils.desktop_size()
         self.setGeometry(
-            screen.width() // 4,
-            screen.height() // 4,
-            screen.width() // 2,
-            screen.height() // 2,
+            width // 4,
+            height // 4,
+            width // 2,
+            height // 2,
         )
 
     def find_git_repo(self):
@@ -293,7 +290,7 @@ class StartupDialog(standard.Dialog):
 
 
 def get_all_repos(context, settings):
-    """Return a sorted list of bookmarks and recent repositorties"""
+    """Return a sorted list of bookmarks and recent repositories"""
     bookmarks = settings.bookmarks
     recent = settings.recent
     all_repos = [(repo, True) for repo in bookmarks] + [
@@ -311,7 +308,7 @@ class BookmarksListView(QtWidgets.QListView):
     """
 
     def __init__(self, context, model, open_selected_repo, set_model, parent=None):
-        super(BookmarksListView, self).__init__(parent)
+        super().__init__(parent)
 
         self.current_mode = ICON_MODE
         self.context = context
@@ -379,7 +376,6 @@ class BookmarksListView(QtWidgets.QListView):
             N_('Remove stale entries for repositories that no longer exist')
         )
 
-        # pylint: disable=no-member
         self.model().itemChanged.connect(self.item_changed)
 
         self.action_group = utils.Group(
@@ -553,7 +549,7 @@ class BookmarksListView(QtWidgets.QListView):
         self.refresh()
 
 
-class BuildItem(object):
+class BuildItem:
     def __init__(self, context):
         self.star_icon = icons.star()
         self.folder_icon = icons.folder()
