@@ -1,6 +1,4 @@
 """Test the cola.gitcmds module"""
-# pylint: disable=redefined-outer-name
-from __future__ import absolute_import, division, print_function, unicode_literals
 import os
 
 from cola import core
@@ -11,7 +9,7 @@ from . import helper
 from .helper import app_context
 
 
-# These assertions make pylint happy. It considers them unused imports otherwise.
+# Prevent unused imports lint errors.
 assert app_context is not None
 
 
@@ -218,6 +216,7 @@ def test_diff_helper(app_context):
         f.write('A change\n')
     helper.run_git('add', 'A')
 
-    expect = '+A change\n'
+    expect_n = '+A change\n'
+    expect_rn = '+A change\r\n'
     actual = gitcmds.diff_helper(app_context, ref='HEAD', cached=True)
-    assert expect in actual
+    assert expect_n in actual or expect_rn in actual
